@@ -20,10 +20,10 @@ from oslo_config import cfg
 from oslo_log import log as logging
 import webob
 
+from neutron._i18n import _, _LI
 from neutron.agent.linux import keepalived
 from neutron.agent.linux import utils as agent_utils
 from neutron.common import utils as common_utils
-from neutron.i18n import _LI
 from neutron.notifiers import batch_notifier
 
 LOG = logging.getLogger(__name__)
@@ -45,6 +45,13 @@ OPTS = [
     cfg.IntOpt('ha_vrrp_advert_int',
                default=2,
                help=_('The advertisement interval in seconds')),
+    cfg.IntOpt('ha_keepalived_state_change_server_threads',
+               default=(1 + common_utils.cpu_count()) // 2,
+               min=1,
+               help=_('Number of concurrent threads for '
+                      'keepalived server connection requests.'
+                      'More threads create a higher CPU load '
+                      'on the agent node.')),
 ]
 
 
@@ -79,7 +86,8 @@ class L3AgentKeepalivedStateChangeServer(object):
 
     def run(self):
         server = agent_utils.UnixDomainWSGIServer(
-            'neutron-keepalived-state-change')
+            'neutron-keepalived-state-change',
+            num_threads=self.conf.ha_keepalived_state_change_server_threads)
         server.start(KeepalivedStateChangeHandler(self.agent),
                      self.get_keepalived_state_change_socket_path(self.conf),
                      workers=0,
@@ -117,13 +125,14 @@ class AgentMixin(object):
 
         try:
             ri = self.router_info[router_id]
-        except AttributeError:
+        except KeyError:
             LOG.info(_LI('Router %s is not managed by this agent. It was '
                          'possibly deleted concurrently.'), router_id)
             return
 
         self._configure_ipv6_ra_on_ext_gw_port_if_necessary(ri, state)
-        self._update_metadata_proxy(ri, router_id, state)
+        if self.conf.enable_metadata_proxy:
+            self._update_metadata_proxy(ri, router_id, state)
         self._update_radvd_daemon(ri, state)
         self.state_change_notifier.queue_event((router_id, state))
 
@@ -133,11 +142,13 @@ class AgentMixin(object):
         # include any IPv6 subnet, enable the gateway interface to accept
         # Router Advts from upstream router for default route.
         ex_gw_port_id = ri.ex_gw_port and ri.ex_gw_port['id']
-        if state == 'master' and ex_gw_port_id and ri.use_ipv6:
-            gateway_ips = ri._get_external_gw_ips(ri.ex_gw_port)
-            if not ri.is_v6_gateway_set(gateway_ips):
-                interface_name = ri.get_external_device_name(ex_gw_port_id)
-                ri.driver.configure_ipv6_ra(ri.ns_name, interface_name)
+        if state == 'master' and ex_gw_port_id:
+            interface_name = ri.get_external_device_name(ex_gw_port_id)
+            if ri.router.get('distributed', False):
+                namespace = ri.ha_namespace
+            else:
+                namespace = ri.ns_name
+            ri._enable_ra_on_gw(ri.ex_gw_port, namespace, interface_name)
 
     def _update_metadata_proxy(self, ri, router_id, state):
         if state == 'master':
