@@ -14,6 +14,12 @@
 
 import os
 
+from oslo_log import log as logging
+
+from neutron._i18n import _LW
+
+LOG = logging.getLogger(__name__)
+
 
 class ItemAllocator(object):
     """Manages allocation of items from a pool
@@ -42,11 +48,21 @@ class ItemAllocator(object):
         self.remembered = {}
         self.pool = item_pool
 
+        read_error = False
         for line in self._read():
-            key, saved_value = line.strip().split(delimiter)
-            self.remembered[key] = self.ItemClass(saved_value)
+            try:
+                key, saved_value = line.strip().split(delimiter)
+                self.remembered[key] = self.ItemClass(saved_value)
+            except ValueError:
+                read_error = True
+                LOG.warning(_LW("Invalid line in %(file)s, "
+                                "ignoring: %(line)s"),
+                            {'file': state_file, 'line': line})
 
         self.pool.difference_update(self.remembered.values())
+        if read_error:
+            LOG.debug("Re-writing file %s due to read error", state_file)
+            self._write_allocations()
 
     def allocate(self, key):
         """Try to allocate an item of ItemClass type.
@@ -65,6 +81,9 @@ class ItemAllocator(object):
         allocations to free the pool.  This final desperate step will not
         happen often in practice.
         """
+        if key in self.allocations:
+            return self.allocations[key]
+
         if key in self.remembered:
             self.allocations[key] = self.remembered.pop(key)
             return self.allocations[key]
@@ -74,7 +93,9 @@ class ItemAllocator(object):
             self.pool.update(self.remembered.values())
             self.remembered.clear()
             if not self.pool:
-                # More than 256 routers on a compute node!
+                # The number of address pairs allocated from the
+                # pool depends upon the prefix length specified
+                # in DVR_FIP_LL_CIDR
                 raise RuntimeError("Cannot allocate item of type:"
                                    " %s from pool using file %s"
                                    % (self.ItemClass, self.state_file))
