@@ -20,32 +20,16 @@ from oslo_config import cfg
 from oslo_log import log as logging
 import oslo_messaging
 
-from neutron._i18n import _, _LI, _LW
+from neutron._i18n import _LI, _LW
 from neutron.agent import firewall
 from neutron.api.rpc.handlers import securitygroups_rpc
+from neutron.conf.agent import securitygroups_rpc as sc_cfg
+
 
 LOG = logging.getLogger(__name__)
 
 
-security_group_opts = [
-    cfg.StrOpt(
-        'firewall_driver',
-        help=_('Driver for security groups firewall in the L2 agent')),
-    cfg.BoolOpt(
-        'enable_security_group',
-        default=True,
-        help=_(
-            'Controls whether the neutron security group API is enabled '
-            'in the server. It should be false when using no security '
-            'groups or using the nova security group API.')),
-    cfg.BoolOpt(
-        'enable_ipset',
-        default=True,
-        help=_('Use ipset to speed-up the iptables based security groups. '
-               'Enabling ipset support requires that ipset is installed on L2 '
-               'agent node.'))
-]
-cfg.CONF.register_opts(security_group_opts, 'SECURITYGROUP')
+sc_cfg.register_securitygroups_opts()
 
 
 #This is backward compatibility check for Havana
@@ -90,7 +74,6 @@ class SecurityGroupAgentRpc(object):
         self.context = context
         self.plugin_rpc = plugin_rpc
         self.init_firewall(defer_refresh_firewall, integration_bridge)
-        self.local_vlan_map = local_vlan_map
 
     def init_firewall(self, defer_refresh_firewall=False,
                       integration_bridge=None):
@@ -152,6 +135,9 @@ class SecurityGroupAgentRpc(object):
         if not device_ids:
             return
         LOG.info(_LI("Preparing filters for devices %s"), device_ids)
+        self._apply_port_filter(device_ids)
+
+    def _apply_port_filter(self, device_ids, update_filter=False):
         if self.use_enhanced_rpc:
             devices_info = self.plugin_rpc.security_group_info_for_devices(
                 self.context, list(device_ids))
@@ -169,7 +155,12 @@ class SecurityGroupAgentRpc(object):
                 self._update_security_group_info(
                     security_groups, security_group_member_ips)
             for device in devices.values():
-                self.firewall.prepare_port_filter(device)
+                if update_filter:
+                    LOG.debug("Update port filter for %s", device['device'])
+                    self.firewall.update_port_filter(device)
+                else:
+                    LOG.debug("Prepare port filter for %s", device['device'])
+                    self.firewall.prepare_port_filter(device)
 
     def _update_security_group_info(self, security_groups,
                                     security_group_member_ips):
@@ -242,25 +233,7 @@ class SecurityGroupAgentRpc(object):
             if not device_ids:
                 LOG.info(_LI("No ports here to refresh firewall"))
                 return
-        if self.use_enhanced_rpc:
-            devices_info = self.plugin_rpc.security_group_info_for_devices(
-                self.context, device_ids)
-            devices = devices_info['devices']
-            security_groups = devices_info['security_groups']
-            security_group_member_ips = devices_info['sg_member_ips']
-        else:
-            devices = self.plugin_rpc.security_group_rules_for_devices(
-                self.context, device_ids)
-
-        with self.firewall.defer_apply():
-            if self.use_enhanced_rpc:
-                LOG.debug("Update security group information for ports %s",
-                          devices.keys())
-                self._update_security_group_info(
-                    security_groups, security_group_member_ips)
-            for device in devices.values():
-                LOG.debug("Update port filter for %s", device['device'])
-                self.firewall.update_port_filter(device)
+        self._apply_port_filter(device_ids, update_filter=True)
 
     def firewall_refresh_needed(self):
         return self.global_refresh_firewall or self.devices_to_refilter
@@ -310,7 +283,7 @@ class SecurityGroupAgentRpc(object):
                 self.refresh_firewall(updated_devices)
 
 
-# TODO(armax): for bw compat with external dependencies; to be dropped in M.
+# TODO(armax): For bw compat with external dependencies; to be dropped in M.
 SG_RPC_VERSION = (
     securitygroups_rpc.SecurityGroupAgentRpcApiMixin.SG_RPC_VERSION
 )
