@@ -27,12 +27,12 @@ def unsubscribe(callback, resource_type):
     _get_manager().unregister(callback, resource_type)
 
 
-def push(resource_type, resource, event_type):
-    """Push resource events into all registered callbacks for the type."""
+def push(resource_type, resource_list, event_type):
+    """Push resource list into all registered callbacks for the event type."""
 
     callbacks = _get_manager().get_callbacks(resource_type)
     for callback in callbacks:
-        callback(resource, event_type)
+        callback(resource_list, event_type)
 
 
 def clear():
