@@ -18,17 +18,13 @@
 import copy
 import datetime
 
-from debtcollector import removals
 from oslo_context import context as oslo_context
-from oslo_log import log as logging
+from oslo_db.sqlalchemy import enginefacade
 
 from neutron.db import api as db_api
 from neutron import policy
 
 
-LOG = logging.getLogger(__name__)
-
-
 class ContextBase(oslo_context.RequestContext):
     """Security context and request information.
 
@@ -36,31 +32,26 @@ class ContextBase(oslo_context.RequestContext):
 
     """
 
-    @removals.removed_kwarg('read_deleted')
-    def __init__(self, user_id, tenant_id, is_admin=None, roles=None,
-                 timestamp=None, request_id=None, tenant_name=None,
-                 user_name=None, overwrite=True, auth_token=None,
+    def __init__(self, user_id=None, tenant_id=None, is_admin=None,
+                 timestamp=None, tenant_name=None, user_name=None,
                  is_advsvc=None, **kwargs):
         """Object initialization.
 
         :param overwrite: Set to False to ensure that the greenthread local
             copy of the index is not overwritten.
-
-        :param kwargs: Extra arguments that might be present, but we ignore
-            because they possibly came in from older rpc messages.
         """
-        super(ContextBase, self).__init__(auth_token=auth_token,
-                                          user=user_id, tenant=tenant_id,
-                                          is_admin=is_admin,
-                                          request_id=request_id,
-                                          overwrite=overwrite)
+        # NOTE(jamielennox): We maintain these arguments in order for tests
+        # that pass arguments positionally.
+        kwargs.setdefault('user', user_id)
+        kwargs.setdefault('tenant', tenant_id)
+        super(ContextBase, self).__init__(is_admin=is_admin, **kwargs)
+
         self.user_name = user_name
         self.tenant_name = tenant_name
 
         if not timestamp:
             timestamp = datetime.datetime.utcnow()
         self.timestamp = timestamp
-        self.roles = roles or []
         self.is_advsvc = is_advsvc
         if self.is_advsvc is None:
             self.is_advsvc = self.is_admin or policy.check_is_advsvc(self)
@@ -93,7 +84,6 @@ class ContextBase(oslo_context.RequestContext):
             'user_id': self.user_id,
             'tenant_id': self.tenant_id,
             'project_id': self.project_id,
-            'roles': self.roles,
             'timestamp': str(self.timestamp),
             'tenant_name': self.tenant_name,
             'project_name': self.tenant_name,
@@ -103,10 +93,17 @@ class ContextBase(oslo_context.RequestContext):
 
     @classmethod
     def from_dict(cls, values):
-        return cls(**values)
+        return cls(user_id=values.get('user_id', values.get('user')),
+                   tenant_id=values.get('tenant_id', values.get('project_id')),
+                   is_admin=values.get('is_admin'),
+                   roles=values.get('roles'),
+                   timestamp=values.get('timestamp'),
+                   request_id=values.get('request_id'),
+                   tenant_name=values.get('tenant_name'),
+                   user_name=values.get('user_name'),
+                   auth_token=values.get('auth_token'))
 
-    @removals.removed_kwarg('read_deleted')
-    def elevated(self, read_deleted=None):
+    def elevated(self):
         """Return a version of this context with admin flag set."""
         context = copy.copy(self)
         context.is_admin = True
@@ -117,29 +114,35 @@ class ContextBase(oslo_context.RequestContext):
         return context
 
 
-class Context(ContextBase):
+@enginefacade.transaction_context_provider
+class ContextBaseWithSession(ContextBase):
+    pass
+
+
+class Context(ContextBaseWithSession):
     def __init__(self, *args, **kwargs):
         super(Context, self).__init__(*args, **kwargs)
         self._session = None
 
     @property
     def session(self):
+        # TODO(akamyshnikova): checking for session attribute won't be needed
+        # when reader and writer will be used
+        if hasattr(super(Context, self), 'session'):
+            return super(Context, self).session
         if self._session is None:
             self._session = db_api.get_session()
         return self._session
 
 
-@removals.removed_kwarg('read_deleted')
-@removals.removed_kwarg('load_admin_roles')
-def get_admin_context(read_deleted="no", load_admin_roles=True):
+def get_admin_context():
     return Context(user_id=None,
                    tenant_id=None,
                    is_admin=True,
                    overwrite=False)
 
 
-@removals.removed_kwarg('read_deleted')
-def get_admin_context_without_session(read_deleted="no"):
+def get_admin_context_without_session():
     return ContextBase(user_id=None,
                        tenant_id=None,
                        is_admin=True)
