@@ -16,8 +16,10 @@
 import datetime
 
 from eventlet import greenthread
+from neutron_lib.api import converters
+from neutron_lib import constants
+from neutron_lib.db import model_base
 from oslo_config import cfg
-from oslo_db import exception as db_exc
 from oslo_log import log as logging
 import oslo_messaging
 from oslo_serialization import jsonutils
@@ -34,10 +36,9 @@ from neutron.api.v2 import attributes
 from neutron.callbacks import events
 from neutron.callbacks import registry
 from neutron.callbacks import resources
-from neutron.common import constants
+from neutron.common import constants as n_const
 from neutron import context
 from neutron.db import api as db_api
-from neutron.db import model_base
 from neutron.extensions import agent as ext_agent
 from neutron.extensions import availability_zone as az_ext
 from neutron import manager
@@ -142,6 +143,7 @@ class AgentAvailabilityZoneMixin(az_ext.AvailabilityZonePluginBase):
             result[key] = agent.admin_state_up or result.get(key, False)
         return result
 
+    @db_api.retry_if_session_inactive()
     def get_availability_zones(self, context, filters=None, fields=None,
                                sorts=None, limit=None, marker=None,
                                page_reverse=False):
@@ -154,6 +156,7 @@ class AgentAvailabilityZoneMixin(az_ext.AvailabilityZonePluginBase):
                 for k, v in six.iteritems(self._list_availability_zones(
                                            context, filters))]
 
+    @db_api.retry_if_session_inactive()
     def validate_availability_zones(self, context, resource_type,
                                     availability_zones):
         """Verify that the availability zones exist."""
@@ -177,11 +180,6 @@ class AgentAvailabilityZoneMixin(az_ext.AvailabilityZonePluginBase):
 class AgentDbMixin(ext_agent.AgentPluginBase, AgentAvailabilityZoneMixin):
     """Mixin class to add agent extension to db_base_plugin_v2."""
 
-    def __init__(self, *args, **kwargs):
-        version_manager.set_consumer_versions_callback(
-            self._get_agents_resource_versions)
-        super(AgentDbMixin, self).__init__(*args, **kwargs)
-
     def _get_agent(self, context, id):
         try:
             agent = self._get_by_id(context, Agent, id)
@@ -189,6 +187,7 @@ class AgentDbMixin(ext_agent.AgentPluginBase, AgentAvailabilityZoneMixin):
             raise ext_agent.AgentNotFound(id=id)
         return agent
 
+    @db_api.retry_if_session_inactive()
     def get_enabled_agent_on_host(self, context, agent_type, host):
         """Return agent of agent_type for the specified host."""
         query = context.session.query(Agent)
@@ -257,6 +256,7 @@ class AgentDbMixin(ext_agent.AgentPluginBase, AgentAvailabilityZoneMixin):
         res['availability_zone'] = agent['availability_zone']
         return self._fields(res, fields)
 
+    @db_api.retry_if_session_inactive()
     def delete_agent(self, context, id):
         agent = self._get_agent(context, id)
         registry.notify(resources.AGENT, events.BEFORE_DELETE, self,
@@ -264,6 +264,7 @@ class AgentDbMixin(ext_agent.AgentPluginBase, AgentAvailabilityZoneMixin):
         with context.session.begin(subtransactions=True):
             context.session.delete(agent)
 
+    @db_api.retry_if_session_inactive()
     def update_agent(self, context, id, agent):
         agent_data = agent['agent']
         with context.session.begin(subtransactions=True):
@@ -271,20 +272,23 @@ class AgentDbMixin(ext_agent.AgentPluginBase, AgentAvailabilityZoneMixin):
             agent.update(agent_data)
         return self._make_agent_dict(agent)
 
+    @db_api.retry_if_session_inactive()
     def get_agents_db(self, context, filters=None):
         query = self._get_collection_query(context, Agent, filters=filters)
         return query.all()
 
+    @db_api.retry_if_session_inactive()
     def get_agents(self, context, filters=None, fields=None):
         agents = self._get_collection(context, Agent,
                                       self._make_agent_dict,
                                       filters=filters, fields=fields)
         alive = filters and filters.get('alive', None)
         if alive:
-            alive = attributes.convert_to_boolean(alive[0])
+            alive = converters.convert_to_boolean(alive[0])
             agents = [agent for agent in agents if agent['alive'] == alive]
         return agents
 
+    @db_api.retry_db_errors
     def agent_health_check(self):
         """Scan agents and log if some are considered dead."""
         agents = self.get_agents(context.get_admin_context(),
@@ -318,10 +322,24 @@ class AgentDbMixin(ext_agent.AgentPluginBase, AgentAvailabilityZoneMixin):
             raise ext_agent.MultipleAgentFoundByTypeHost(agent_type=agent_type,
                                                          host=host)
 
+    @db_api.retry_if_session_inactive()
     def get_agent(self, context, id, fields=None):
         agent = self._get_agent(context, id)
         return self._make_agent_dict(agent, fields)
 
+    @db_api.retry_if_session_inactive()
+    def filter_hosts_with_network_access(
+            self, context, network_id, candidate_hosts):
+        """Filter hosts with access to network_id.
+
+        This method returns a subset of candidate_hosts with the ones with
+        network access to network_id.
+
+        A plugin can overload this method to define its own host network_id
+        based filter.
+        """
+        return candidate_hosts
+
     def _log_heartbeat(self, state, agent_db, agent_conf):
         if agent_conf.get('log_agent_heartbeats'):
             delta = timeutils.utcnow() - agent_db.heartbeat_timestamp
@@ -332,13 +350,15 @@ class AgentDbMixin(ext_agent.AgentPluginBase, AgentAvailabilityZoneMixin):
                       'uuid': state.get('uuid'),
                       'delta': delta})
 
-    def _create_or_update_agent(self, context, agent_state):
+    @db_api.retry_if_session_inactive()
+    def create_or_update_agent(self, context, agent_state):
         """Registers new agent in the database or updates existing.
 
-        Returns agent status from server point of view: alive, new or revived.
+        Returns tuple of agent status and state.
+        Status is from server point of view: alive, new or revived.
         It could be used by agent to do some sync with the server if needed.
         """
-        status = constants.AGENT_ALIVE
+        status = n_const.AGENT_ALIVE
         with context.session.begin(subtransactions=True):
             res_keys = ['agent_type', 'binary', 'host', 'topic']
             res = dict((k, agent_state[k]) for k in res_keys)
@@ -356,7 +376,7 @@ class AgentDbMixin(ext_agent.AgentPluginBase, AgentAvailabilityZoneMixin):
                 agent_db = self._get_agent_by_type_and_host(
                     context, agent_state['agent_type'], agent_state['host'])
                 if not agent_db.is_active:
-                    status = constants.AGENT_REVIVED
+                    status = n_const.AGENT_REVIVED
                     if 'resource_versions' not in agent_state:
                         # updating agent_state with resource_versions taken
                         # from db so that
@@ -370,6 +390,7 @@ class AgentDbMixin(ext_agent.AgentPluginBase, AgentAvailabilityZoneMixin):
                 greenthread.sleep(0)
                 self._log_heartbeat(agent_state, agent_db, configurations_dict)
                 agent_db.update(res)
+                event_type = events.AFTER_UPDATE
             except ext_agent.AgentNotFoundByTypeHost:
                 greenthread.sleep(0)
                 res['created_at'] = current_time
@@ -379,43 +400,30 @@ class AgentDbMixin(ext_agent.AgentPluginBase, AgentAvailabilityZoneMixin):
                 agent_db = Agent(**res)
                 greenthread.sleep(0)
                 context.session.add(agent_db)
+                event_type = events.AFTER_CREATE
                 self._log_heartbeat(agent_state, agent_db, configurations_dict)
-                status = constants.AGENT_NEW
+                status = n_const.AGENT_NEW
             greenthread.sleep(0)
-        return status
 
-    def create_or_update_agent(self, context, agent):
-        """Create or update agent according to report."""
-        try:
-            return self._create_or_update_agent(context, agent)
-        except db_exc.DBDuplicateEntry:
-            # It might happen that two or more concurrent transactions
-            # are trying to insert new rows having the same value of
-            # (agent_type, host) pair at the same time (if there has
-            # been no such entry in the table and multiple agent status
-            # updates are being processed at the moment). In this case
-            # having a unique constraint on (agent_type, host) columns
-            # guarantees that only one transaction will succeed and
-            # insert a new agent entry, others will fail and be rolled
-            # back. That means we must retry them one more time: no
-            # INSERTs will be issued, because
-            # _get_agent_by_type_and_host() will return the existing
-            # agent entry, which will be updated multiple times
-            return self._create_or_update_agent(context, agent)
+        registry.notify(resources.AGENT, event_type, self, context=context,
+                        host=agent_state['host'], plugin=self,
+                        agent=agent_state)
+        return status, agent_state
 
     def _get_agents_considered_for_versions(self):
         up_agents = self.get_agents(context.get_admin_context(),
                                     filters={'admin_state_up': [True]})
         return filter(self.is_agent_considered_for_versions, up_agents)
 
-    def _get_agents_resource_versions(self, tracker):
+    def get_agents_resource_versions(self, tracker):
         """Get the known agent resource versions and update the tracker.
 
-        Receives a version_manager.ResourceConsumerTracker instance and it's
-        expected to look up in to the database and update every agent resource
-        versions.
+        This function looks up into the database and updates every agent
+        resource versions.
         This method is called from version_manager when the cached information
         has passed TTL.
+
+        :param tracker: receives a version_manager.ResourceConsumerTracker
         """
         for agent in self._get_agents_considered_for_versions():
             resource_versions = agent.get('resource_versions', {})
@@ -440,7 +448,7 @@ class AgentExtRpcCallback(object):
     """
 
     target = oslo_messaging.Target(version='1.1',
-                                   namespace=constants.RPC_NAMESPACE_STATE)
+                                   namespace=n_const.RPC_NAMESPACE_STATE)
     START_TIME = timeutils.utcnow()
 
     def __init__(self, plugin=None):
@@ -453,7 +461,7 @@ class AgentExtRpcCallback(object):
         # Initialize RPC api directed to other neutron-servers
         self.server_versions_rpc = resources_rpc.ResourcesPushToServersRpcApi()
 
-    @db_api.retry_db_errors
+    @db_api.retry_if_session_inactive()
     def report_state(self, context, **kwargs):
         """Report state from agent to server.
 
@@ -473,7 +481,8 @@ class AgentExtRpcCallback(object):
             return
         if not self.plugin:
             self.plugin = manager.NeutronManager.get_plugin()
-        agent_status = self.plugin.create_or_update_agent(context, agent_state)
+        agent_status, agent_state = self.plugin.create_or_update_agent(
+            context, agent_state)
         self._update_local_agent_resource_versions(context, agent_state)
         return agent_status
 
