@@ -13,22 +13,24 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import copy
+
 import netaddr
+from neutron_lib import constants
+from neutron_lib import exceptions as n_exc
 from oslo_db import exception as db_exc
 from oslo_log import log as logging
 from oslo_utils import excutils
 from sqlalchemy import and_
 
-from neutron.api.v2 import attributes
-from neutron.common import constants
-from neutron.common import exceptions as n_exc
+from neutron._i18n import _, _LE, _LW
+from neutron.common import constants as n_const
 from neutron.common import ipv6_utils
 from neutron.db import ipam_backend_mixin
 from neutron.db import models_v2
-from neutron.i18n import _LE
+from neutron.extensions import portbindings
 from neutron.ipam import driver
 from neutron.ipam import exceptions as ipam_exc
-from neutron.ipam import requests as ipam_req
 
 
 LOG = logging.getLogger(__name__)
@@ -41,6 +43,17 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
         return (ip_dict['ip_address'] for ip_dict in all_ips
                 if ip_dict['ip_address'] not in ips_list)
 
+    def _safe_rollback(self, func, *args, **kwargs):
+        """Calls rollback actions and catch all exceptions.
+
+        All exceptions are catched and logged here to prevent rewriting
+        original exception that triggered rollback action.
+        """
+        try:
+            func(*args, **kwargs)
+        except Exception as e:
+            LOG.warning(_LW("Revert failed with: %s"), e)
+
     def _ipam_deallocate_ips(self, context, ipam_driver, port, ips,
                              revert_on_fail=True):
         """Deallocate set of ips over IPAM.
@@ -61,11 +74,18 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
                               ip)
         except Exception:
             with excutils.save_and_reraise_exception():
+                if not ipam_driver.needs_rollback():
+                    return
+
                 LOG.debug("An exception occurred during IP deallocation.")
                 if revert_on_fail and deallocated:
                     LOG.debug("Reverting deallocation")
-                    self._ipam_allocate_ips(context, ipam_driver, port,
-                                            deallocated, revert_on_fail=False)
+                    # In case of deadlock allocate fails with db error
+                    # and rewrites original exception preventing db_retry
+                    # wrappers from restarting entire api request.
+                    self._safe_rollback(self._ipam_allocate_ips, context,
+                                        ipam_driver, port, deallocated,
+                                        revert_on_fail=False)
                 elif not revert_on_fail and ips:
                     addresses = ', '.join(self._get_failed_ips(ips,
                                                                deallocated))
@@ -73,28 +93,6 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
                                   "external system for %s"), addresses)
         return deallocated
 
-    def _ipam_try_allocate_ip(self, context, ipam_driver, port, ip_dict):
-        factory = ipam_driver.get_address_request_factory()
-        ip_request = factory.get_request(context, port, ip_dict)
-        ipam_subnet = ipam_driver.get_subnet(ip_dict['subnet_id'])
-        return ipam_subnet.allocate(ip_request)
-
-    def _ipam_allocate_single_ip(self, context, ipam_driver, port, subnets):
-        """Allocates single ip from set of subnets
-
-        Raises n_exc.IpAddressGenerationFailure if allocation failed for
-        all subnets.
-        """
-        for subnet in subnets:
-            try:
-                return [self._ipam_try_allocate_ip(context, ipam_driver,
-                                                   port, subnet),
-                        subnet]
-            except ipam_exc.IpAddressGenerationFailure:
-                continue
-        raise n_exc.IpAddressGenerationFailure(
-            net_id=port['network_id'])
-
     def _ipam_allocate_ips(self, context, ipam_driver, port, ips,
                            revert_on_fail=True):
         """Allocate set of ips over IPAM.
@@ -113,21 +111,35 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
                 # By default IP info is dict, used to allocate single ip
                 # from single subnet.
                 # IP info can be list, used to allocate single ip from
-                # multiple subnets (i.e. first successful ip allocation
-                # is returned)
+                # multiple subnets
                 ip_list = [ip] if isinstance(ip, dict) else ip
-                ip_address, ip_subnet = self._ipam_allocate_single_ip(
-                    context, ipam_driver, port, ip_list)
+                subnets = [ip_dict['subnet_id'] for ip_dict in ip_list]
+                try:
+                    factory = ipam_driver.get_address_request_factory()
+                    ip_request = factory.get_request(context, port, ip_list[0])
+                    ipam_allocator = ipam_driver.get_allocator(subnets)
+                    ip_address, subnet_id = ipam_allocator.allocate(ip_request)
+                except ipam_exc.IpAddressGenerationFailureAllSubnets:
+                    raise n_exc.IpAddressGenerationFailure(
+                        net_id=port['network_id'])
+
                 allocated.append({'ip_address': ip_address,
-                                  'subnet_id': ip_subnet['subnet_id']})
+                                  'subnet_id': subnet_id})
         except Exception:
             with excutils.save_and_reraise_exception():
+                if not ipam_driver.needs_rollback():
+                    return
+
                 LOG.debug("An exception occurred during IP allocation.")
 
                 if revert_on_fail and allocated:
                     LOG.debug("Reverting allocation")
-                    self._ipam_deallocate_ips(context, ipam_driver, port,
-                                              allocated, revert_on_fail=False)
+                    # In case of deadlock deallocation fails with db error
+                    # and rewrites original exception preventing db_retry
+                    # wrappers from restarting entire api request.
+                    self._safe_rollback(self._ipam_deallocate_ips, context,
+                                        ipam_driver, port, allocated,
+                                        revert_on_fail=False)
                 elif not revert_on_fail and ips:
                     addresses = ', '.join(self._get_failed_ips(ips,
                                                                allocated))
@@ -137,9 +149,6 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
         return allocated
 
     def _ipam_update_allocation_pools(self, context, ipam_driver, subnet):
-        self.validate_allocation_pools(subnet['allocation_pools'],
-                                       subnet['cidr'])
-
         factory = ipam_driver.get_subnet_request_factory()
         subnet_request = factory.get_request(context, subnet, None)
 
@@ -150,10 +159,17 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
         ipam_driver.remove_subnet(subnet_id)
 
     def allocate_ips_for_port_and_store(self, context, port, port_id):
-        network_id = port['port']['network_id']
+        # Make a copy of port dict to prevent changing
+        # incoming dict by adding 'id' to it.
+        # Deepcopy doesn't work correctly in this case, because copy of
+        # ATTR_NOT_SPECIFIED object happens. Address of copied object doesn't
+        # match original object, so 'is' check fails
+        port_copy = {'port': port['port'].copy()}
+        port_copy['port']['id'] = port_id
+        network_id = port_copy['port']['network_id']
         ips = []
         try:
-            ips = self._allocate_ips_for_port(context, port)
+            ips = self._allocate_ips_for_port(context, port_copy)
             for ip in ips:
                 ip_address = ip['ip_address']
                 subnet_id = ip['subnet_id']
@@ -164,12 +180,15 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
         except Exception:
             with excutils.save_and_reraise_exception():
                 if ips:
-                    LOG.debug("An exception occurred during port creation."
-                              "Reverting IP allocation")
                     ipam_driver = driver.Pool.get_instance(None, context)
-                    self._ipam_deallocate_ips(context, ipam_driver,
-                                              port['port'], ips,
-                                              revert_on_fail=False)
+                    if not ipam_driver.needs_rollback():
+                        return
+
+                    LOG.debug("An exception occurred during port creation. "
+                              "Reverting IP allocation")
+                    self._safe_rollback(self._ipam_deallocate_ips, context,
+                                        ipam_driver, port_copy['port'], ips,
+                                        revert_on_fail=False)
 
     def _allocate_ips_for_port(self, context, port):
         """Allocate IP addresses for the port. IPAM version.
@@ -179,56 +198,44 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
         a subnet_id then allocate an IP address accordingly.
         """
         p = port['port']
-        ips = []
-        v6_stateless = []
-        net_id_filter = {'network_id': [p['network_id']]}
-        subnets = self._get_subnets(context, filters=net_id_filter)
-        is_router_port = (
-            p['device_owner'] in constants.ROUTER_INTERFACE_OWNERS_SNAT)
+        subnets = self._ipam_get_subnets(context,
+                                         network_id=p['network_id'],
+                                         host=p.get(portbindings.HOST_ID),
+                                         service_type=p.get('device_owner'))
 
-        fixed_configured = p['fixed_ips'] is not attributes.ATTR_NOT_SPECIFIED
+        v4, v6_stateful, v6_stateless = self._classify_subnets(
+            context, subnets)
+
+        fixed_configured = p['fixed_ips'] is not constants.ATTR_NOT_SPECIFIED
         if fixed_configured:
             ips = self._test_fixed_ips_for_port(context,
                                                 p["network_id"],
                                                 p['fixed_ips'],
-                                                p['device_owner'])
-            # For ports that are not router ports, implicitly include all
-            # auto-address subnets for address association.
-            if not is_router_port:
-                v6_stateless += [subnet for subnet in subnets
-                                 if ipv6_utils.is_auto_address_subnet(subnet)]
+                                                p['device_owner'],
+                                                subnets)
         else:
-            # Split into v4, v6 stateless and v6 stateful subnets
-            v4 = []
-            v6_stateful = []
-            for subnet in subnets:
-                if subnet['ip_version'] == 4:
-                    v4.append(subnet)
-                else:
-                    if ipv6_utils.is_auto_address_subnet(subnet):
-                        if not is_router_port:
-                            v6_stateless.append(subnet)
-                    else:
-                        v6_stateful.append(subnet)
-
+            ips = []
             version_subnets = [v4, v6_stateful]
             for subnets in version_subnets:
                 if subnets:
                     ips.append([{'subnet_id': s['id']}
                                 for s in subnets])
 
-        for subnet in v6_stateless:
-            # IP addresses for IPv6 SLAAC and DHCPv6-stateless subnets
-            # are implicitly included.
-            ips.append({'subnet_id': subnet['id'],
-                        'subnet_cidr': subnet['cidr'],
-                        'eui64_address': True,
-                        'mac': p['mac_address']})
+        is_router_port = (
+            p['device_owner'] in constants.ROUTER_INTERFACE_OWNERS_SNAT)
+        if not is_router_port:
+            for subnet in v6_stateless:
+                # IP addresses for IPv6 SLAAC and DHCPv6-stateless subnets
+                # are implicitly included.
+                ips.append({'subnet_id': subnet['id'],
+                            'subnet_cidr': subnet['cidr'],
+                            'eui64_address': True,
+                            'mac': p['mac_address']})
         ipam_driver = driver.Pool.get_instance(None, context)
         return self._ipam_allocate_ips(context, ipam_driver, p, ips)
 
     def _test_fixed_ips_for_port(self, context, network_id, fixed_ips,
-                                 device_owner):
+                                 device_owner, subnets):
         """Test fixed IPs for port.
 
         Check that configured subnets are valid prior to allocating any
@@ -240,10 +247,11 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
         """
         fixed_ip_list = []
         for fixed in fixed_ips:
-            subnet = self._get_subnet_for_fixed_ip(context, fixed, network_id)
+            subnet = self._get_subnet_for_fixed_ip(context, fixed, subnets)
 
             is_auto_addr_subnet = ipv6_utils.is_auto_address_subnet(subnet)
-            if 'ip_address' in fixed:
+            if ('ip_address' in fixed and
+                    subnet['cidr'] != n_const.PROVISIONAL_IPV6_PD_PREFIX):
                 if (is_auto_addr_subnet and device_owner not in
                         constants.ROUTER_INTERFACE_OWNERS):
                     msg = (_("IPv6 address %(address)s can not be directly "
@@ -263,20 +271,30 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
                         not is_auto_addr_subnet):
                     fixed_ip_list.append({'subnet_id': subnet['id']})
 
-        self._validate_max_ips_per_port(fixed_ip_list)
+        self._validate_max_ips_per_port(fixed_ip_list, device_owner)
         return fixed_ip_list
 
-    def _update_ips_for_port(self, context, port,
+    def _update_ips_for_port(self, context, port, host,
                              original_ips, new_ips, mac):
         """Add or remove IPs from the port. IPAM version"""
         added = []
         removed = []
         changes = self._get_changed_ips_for_port(
             context, original_ips, new_ips, port['device_owner'])
+        try:
+            subnets = self._ipam_get_subnets(
+                context, network_id=port['network_id'], host=host)
+        except ipam_exc.DeferIpam:
+            subnets = []
+
         # Check if the IP's to add are OK
         to_add = self._test_fixed_ips_for_port(
             context, port['network_id'], changes.add,
-            port['device_owner'])
+            port['device_owner'], subnets)
+
+        if port['device_owner'] not in constants.ROUTER_INTERFACE_OWNERS:
+            to_add += self._update_ips_for_pd_subnet(
+                context, subnets, changes.add, mac)
 
         ipam_driver = driver.Pool.get_instance(None, context)
         if changes.remove:
@@ -284,7 +302,7 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
                                                 changes.remove)
         if to_add:
             added = self._ipam_allocate_ips(context, ipam_driver,
-                                            changes, to_add)
+                                            port, to_add)
         return self.Changes(add=added,
                             original=changes.original,
                             remove=removed)
@@ -298,7 +316,7 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
                                                  last_ip=last_ip)
             context.session.add(ip_pool)
 
-    def update_port_with_ips(self, context, db_port, new_port, new_mac):
+    def update_port_with_ips(self, context, host, db_port, new_port, new_mac):
         changes = self.Changes(add=[], original=[], remove=[])
 
         if 'fixed_ips' in new_port:
@@ -306,36 +324,51 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
                                             process_extensions=False)
             changes = self._update_ips_for_port(context,
                                                 db_port,
+                                                host,
                                                 original["fixed_ips"],
                                                 new_port['fixed_ips'],
                                                 new_mac)
         try:
+            # Expire the fixed_ips of db_port in current transaction, because
+            # it will be changed in the following operation and the latest
+            # data is expected.
+            context.session.expire(db_port, ['fixed_ips'])
+
             # Check if the IPs need to be updated
             network_id = db_port['network_id']
+            for ip in changes.remove:
+                self._delete_ip_allocation(context, network_id,
+                                           ip['subnet_id'], ip['ip_address'])
             for ip in changes.add:
                 self._store_ip_allocation(
                     context, ip['ip_address'], network_id,
                     ip['subnet_id'], db_port.id)
-            for ip in changes.remove:
-                self._delete_ip_allocation(context, network_id,
-                                           ip['subnet_id'], ip['ip_address'])
             self._update_db_port(context, db_port, new_port, network_id,
                                  new_mac)
         except Exception:
             with excutils.save_and_reraise_exception():
                 if 'fixed_ips' in new_port:
-                    LOG.debug("An exception occurred during port update.")
                     ipam_driver = driver.Pool.get_instance(None, context)
+                    if not ipam_driver.needs_rollback():
+                        return
+
+                    LOG.debug("An exception occurred during port update.")
                     if changes.add:
                         LOG.debug("Reverting IP allocation.")
-                        self._ipam_deallocate_ips(context, ipam_driver,
-                                                  db_port, changes.add,
-                                                  revert_on_fail=False)
+                        self._safe_rollback(self._ipam_deallocate_ips,
+                                            context,
+                                            ipam_driver,
+                                            db_port,
+                                            changes.add,
+                                            revert_on_fail=False)
                     if changes.remove:
                         LOG.debug("Reverting IP deallocation.")
-                        self._ipam_allocate_ips(context, ipam_driver,
-                                                db_port, changes.remove,
-                                                revert_on_fail=False)
+                        self._safe_rollback(self._ipam_allocate_ips,
+                                            context,
+                                            ipam_driver,
+                                            db_port,
+                                            changes.remove,
+                                            revert_on_fail=False)
         return changes
 
     def delete_port(self, context, id):
@@ -351,22 +384,22 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
                                   port['fixed_ips'])
 
     def update_db_subnet(self, context, id, s, old_pools):
+        # 'allocation_pools' is removed from 's' in
+        # _update_subnet_allocation_pools (ipam_backend_mixin),
+        # so create unchanged copy for ipam driver
+        subnet_copy = copy.deepcopy(s)
+        subnet, changes = super(IpamPluggableBackend, self).update_db_subnet(
+            context, id, s, old_pools)
         ipam_driver = driver.Pool.get_instance(None, context)
-        if "allocation_pools" in s:
-            self._ipam_update_allocation_pools(context, ipam_driver, s)
 
-        try:
-            subnet, changes = super(IpamPluggableBackend,
-                                    self).update_db_subnet(context, id,
-                                                           s, old_pools)
-        except Exception:
-            with excutils.save_and_reraise_exception():
-                if "allocation_pools" in s and old_pools:
-                    LOG.error(
-                        _LE("An exception occurred during subnet update."
-                            "Reverting allocation pool changes"))
-                    s['allocation_pools'] = old_pools
-                    self._ipam_update_allocation_pools(context, ipam_driver, s)
+        # Set old allocation pools if no new pools are provided by user.
+        # Passing old pools allows to call ipam driver on each subnet update
+        # even if allocation pools are not changed. So custom ipam drivers
+        # are able to track other fields changes on subnet update.
+        if 'allocation_pools' not in subnet_copy:
+            subnet_copy['allocation_pools'] = old_pools
+        self._ipam_update_allocation_pools(context, ipam_driver, subnet_copy)
+
         return subnet, changes
 
     def add_auto_addrs_on_network_ports(self, context, subnet, ipam_subnet):
@@ -378,10 +411,15 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
                 and_(models_v2.Port.network_id == network_id,
                      ~models_v2.Port.device_owner.in_(
                          constants.ROUTER_INTERFACE_OWNERS_SNAT)))
+            updated_ports = []
+            ipam_driver = driver.Pool.get_instance(None, context)
+            factory = ipam_driver.get_address_request_factory()
             for port in ports:
-                ip_request = ipam_req.AutomaticAddressRequest(
-                    prefix=subnet['cidr'],
-                    mac=port['mac_address'])
+                ip = {'subnet_id': subnet['id'],
+                      'subnet_cidr': subnet['cidr'],
+                      'eui64_address': True,
+                      'mac': port['mac_address']}
+                ip_request = factory.get_request(context, port, ip)
                 ip_address = ipam_subnet.allocate(ip_request)
                 allocated = models_v2.IPAllocation(network_id=network_id,
                                                    port_id=port['id'],
@@ -394,6 +432,7 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
                     # the corresponding port has been deleted.
                     with context.session.begin_nested():
                         context.session.add(allocated)
+                    updated_ports.append(port['id'])
                 except db_exc.DBReferenceError:
                     LOG.debug("Port %s was deleted while updating it with an "
                               "IPv6 auto-address. Ignoring.", port['id'])
@@ -404,17 +443,18 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
                     except Exception:
                         LOG.debug("Reverting IP allocation failed for %s",
                                   ip_address)
+            return updated_ports
 
     def allocate_subnet(self, context, network, subnet, subnetpool_id):
         subnetpool = None
 
         if subnetpool_id and not subnetpool_id == constants.IPV6_PD_POOL_ID:
-            subnetpool = self._get_subnetpool(context, subnetpool_id)
+            subnetpool = self._get_subnetpool(context, id=subnetpool_id)
             self._validate_ip_version_with_subnetpool(subnet, subnetpool)
 
         # gateway_ip and allocation pools should be validated or generated
         # only for specific request
-        if subnet['cidr'] is not attributes.ATTR_NOT_SPECIFIED:
+        if subnet['cidr'] is not constants.ATTR_NOT_SPECIFIED:
             subnet['gateway_ip'] = self._gateway_ip_str(subnet,
                                                         subnet['cidr'])
             subnet['allocation_pools'] = self._prepare_allocation_pools(
@@ -446,7 +486,12 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
             # IPAM part rolled back in exception handling
             # and subnet part is rolled back by transaction rollback.
             with excutils.save_and_reraise_exception():
-                LOG.debug("An exception occurred during subnet creation."
+                if not ipam_driver.needs_rollback():
+                    return
+
+                LOG.debug("An exception occurred during subnet creation. "
                           "Reverting subnet allocation.")
-                self.delete_subnet(context, subnet_request.subnet_id)
+                self._safe_rollback(self.delete_subnet,
+                                    context,
+                                    subnet_request.subnet_id)
         return subnet, ipam_subnet
