@@ -16,27 +16,32 @@
 import functools
 
 import netaddr
+from neutron_lib.api import validators
+from neutron_lib import constants
+from neutron_lib.db import model_base
+from neutron_lib import exceptions as n_exc
 from oslo_config import cfg
 from oslo_db import exception as db_exc
 from oslo_log import helpers as log_helpers
 from oslo_log import log as logging
 from oslo_utils import excutils
+import six
 import sqlalchemy as sa
 from sqlalchemy import exc as sql_exc
 from sqlalchemy import orm
 
 from neutron._i18n import _, _LI
 from neutron.api.v2 import attributes
-from neutron.common import constants
-from neutron.common import exceptions as n_exc
+from neutron.common import constants as n_const
 from neutron.common import utils as n_utils
 from neutron.db import agents_db
+from neutron.db import api as db_api
 from neutron.db.availability_zone import router as router_az_db
 from neutron.db import common_db_mixin
 from neutron.db import l3_attrs_db
 from neutron.db import l3_db
 from neutron.db import l3_dvr_db
-from neutron.db import model_base
+from neutron.db.l3_dvr_db import is_distributed_router
 from neutron.db import models_v2
 from neutron.extensions import l3
 from neutron.extensions import l3_ext_ha_mode as l3_ha
@@ -61,12 +66,16 @@ L3_HA_OPTS = [
                       "scheduled on. If it is set to 0 then the router will "
                       "be scheduled on every agent.")),
     cfg.IntOpt('min_l3_agents_per_router',
-               default=constants.MINIMUM_AGENTS_FOR_HA,
-               help=_("Minimum number of L3 agents which a HA router will be "
-                      "scheduled on. If it is set to 0 then the router will "
-                      "be scheduled on every agent.")),
+               default=n_const.DEFAULT_MINIMUM_AGENTS_FOR_HA,
+               help=_("DEPRECATED: Minimum number of L3 agents that have to "
+                      "be available in order to allow a new HA router to be "
+                      "scheduled. This option is deprecated in the Newton "
+                      "release and will be removed for the Ocata release "
+                      "where the scheduling of new HA routers will always "
+                      "be allowed."),
+               deprecated_for_removal=True),
     cfg.StrOpt('l3_ha_net_cidr',
-               default='169.254.192.0/18',
+               default=n_const.L3_HA_NET_CIDR,
                help=_('Subnet used for the l3 HA admin network.')),
     cfg.StrOpt('l3_ha_network_type', default='',
                help=_("The network type to use when creating the HA network "
@@ -110,14 +119,14 @@ class L3HARouterAgentPortBinding(model_base.BASEV2):
                                           ondelete='CASCADE'))
     agent = orm.relationship(agents_db.Agent)
 
-    state = sa.Column(sa.Enum(constants.HA_ROUTER_STATE_ACTIVE,
-                              constants.HA_ROUTER_STATE_STANDBY,
+    state = sa.Column(sa.Enum(n_const.HA_ROUTER_STATE_ACTIVE,
+                              n_const.HA_ROUTER_STATE_STANDBY,
                               name='l3_ha_states'),
-                      default=constants.HA_ROUTER_STATE_STANDBY,
-                      server_default=constants.HA_ROUTER_STATE_STANDBY)
+                      default=n_const.HA_ROUTER_STATE_STANDBY,
+                      server_default=n_const.HA_ROUTER_STATE_STANDBY)
 
 
-class L3HARouterNetwork(model_base.BASEV2):
+class L3HARouterNetwork(model_base.BASEV2, model_base.HasProjectPrimaryKey):
     """Host HA network for a tenant.
 
     One HA Network is used per tenant, all HA router ports are created
@@ -126,8 +135,6 @@ class L3HARouterNetwork(model_base.BASEV2):
 
     __tablename__ = 'ha_router_networks'
 
-    tenant_id = sa.Column(sa.String(attributes.TENANT_ID_MAX_LEN),
-                          primary_key=True, nullable=False)
     network_id = sa.Column(sa.String(36),
                            sa.ForeignKey('networks.id', ondelete="CASCADE"),
                            nullable=False, primary_key=True)
@@ -178,7 +185,7 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
             raise l3_ha.HAMaximumAgentsNumberNotValid(
                 max_agents=max_agents, min_agents=min_agents)
 
-        if min_agents < constants.MINIMUM_AGENTS_FOR_HA:
+        if min_agents < n_const.MINIMUM_MINIMUM_AGENTS_FOR_HA:
             raise l3_ha.HAMinimumAgentsNumberNotValid()
 
     def __init__(self):
@@ -199,7 +206,10 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
 
         return allocated_vr_ids
 
+    @db_api.retry_if_session_inactive()
     def _allocate_vr_id(self, context, network_id, router_id):
+        # TODO(kevinbenton): let decorator handle duplicate retry
+        # like in review.openstack.org/#/c/367179/1/neutron/db/l3_hamode_db.py
         for count in range(MAX_ALLOCATION_TRIES):
             try:
                 # NOTE(kevinbenton): we disallow subtransactions because the
@@ -230,6 +240,7 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
             network_id=network_id, router_id=router_id,
             max_tries=MAX_ALLOCATION_TRIES)
 
+    @db_api.retry_if_session_inactive()
     def _delete_vr_id_allocation(self, context, ha_network, vr_id):
         with context.session.begin(subtransactions=True):
             context.session.query(L3HARouterVRIdAllocation).filter_by(
@@ -243,7 +254,7 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
     def _create_ha_subnet(self, context, network_id, tenant_id):
         args = {'network_id': network_id,
                 'tenant_id': '',
-                'name': constants.HA_SUBNET_NAME % tenant_id,
+                'name': n_const.HA_SUBNET_NAME % tenant_id,
                 'ip_version': 4,
                 'cidr': cfg.CONF.l3_ha_net_cidr,
                 'enable_dhcp': False,
@@ -282,7 +293,7 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
         admin_ctx = context.elevated()
 
         args = {'network':
-                {'name': constants.HA_NETWORK_NAME % tenant_id,
+                {'name': n_const.HA_NETWORK_NAME % tenant_id,
                  'tenant_id': '',
                  'shared': False,
                  'admin_state_up': True}}
@@ -334,6 +345,10 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
     def _create_ha_port_binding(self, context, router_id, port_id):
         try:
             with context.session.begin():
+                routerportbinding = l3_db.RouterPort(
+                    port_id=port_id, router_id=router_id,
+                    port_type=constants.DEVICE_OWNER_ROUTER_HA_INTF)
+                context.session.add(routerportbinding)
                 portbinding = L3HARouterAgentPortBinding(port_id=port_id,
                                                          router_id=router_id)
                 context.session.add(portbinding)
@@ -364,7 +379,7 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
                 'admin_state_up': True,
                 'device_id': router_id,
                 'device_owner': constants.DEVICE_OWNER_ROUTER_HA_INTF,
-                'name': constants.HA_PORT_NAME % tenant_id}
+                'name': n_const.HA_PORT_NAME % tenant_id}
         creation = functools.partial(p_utils.create_port, self._core_plugin,
                                      context, {'port': args})
         content = functools.partial(self._create_ha_port_binding, context,
@@ -423,10 +438,20 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
     @classmethod
     def _is_ha(cls, router):
         ha = router.get('ha')
-        if not attributes.is_attr_set(ha):
+        if not validators.is_attr_set(ha):
             ha = cfg.CONF.l3_ha
         return ha
 
+    def _get_device_owner(self, context, router=None):
+        """Get device_owner for the specified router."""
+        router_is_uuid = isinstance(router, six.string_types)
+        if router_is_uuid:
+            router = self._get_router(context, router)
+        if is_ha_router(router) and not is_distributed_router(router):
+            return constants.DEVICE_OWNER_HA_REPLICATED_INT
+        return super(L3_HA_NAT_db_mixin,
+                     self)._get_device_owner(context, router)
+
     @n_utils.transaction_guard
     def _create_ha_interfaces_and_ensure_network(self, context, router_db):
         """Attach interfaces to a network while tolerating network deletes."""
@@ -436,21 +461,27 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
                                        context, router_db.tenant_id)
         dep_creator = functools.partial(self._create_ha_network,
                                         context, router_db.tenant_id)
+        dep_deleter = functools.partial(self._delete_ha_network, context)
         dep_id_attr = 'network_id'
         return n_utils.create_object_with_dependency(
-            creator, dep_getter, dep_creator, dep_id_attr)
+            creator, dep_getter, dep_creator, dep_id_attr, dep_deleter)
 
+    def _process_extra_attr_router_create(self, context, router_db,
+                                          router_res):
+        router_res['ha'] = self._is_ha(router_res)
+        super(L3_HA_NAT_db_mixin, self)._process_extra_attr_router_create(
+            context, router_db, router_res)
+
+    @db_api.retry_if_session_inactive()
     def create_router(self, context, router):
         is_ha = self._is_ha(router['router'])
-        router['router']['ha'] = is_ha
         if is_ha:
             # we set the allocating status to hide it from the L3 agents
             # until we have created all of the requisite interfaces/networks
-            router['router']['status'] = constants.ROUTER_STATUS_ALLOCATING
+            router['router']['status'] = n_const.ROUTER_STATUS_ALLOCATING
 
         router_dict = super(L3_HA_NAT_db_mixin,
                             self).create_router(context, router)
-
         if is_ha:
             try:
                 router_db = self._get_router(context, router_dict['id'])
@@ -465,7 +496,7 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
                 self.schedule_router(context, router_dict['id'])
                 router_dict['status'] = self._update_router_db(
                     context, router_dict['id'],
-                    {'status': constants.ROUTER_STATUS_ACTIVE})['status']
+                    {'status': n_const.ROUTER_STATUS_ACTIVE})['status']
                 self._notify_ha_interfaces_updated(context, router_db.id,
                                                    schedule_routers=False)
             except Exception:
@@ -473,6 +504,7 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
                     self.delete_router(context, router_dict['id'])
         return router_dict
 
+    @db_api.retry_if_session_inactive()
     def _update_router_db(self, context, router_id, data):
         router_db = self._get_router(context, router_id)
 
@@ -517,7 +549,7 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
             # Keep in mind that if we want conversion to be hitless, this
             # status cannot be used because agents treat hidden routers as
             # deleted routers.
-            data['status'] = constants.ROUTER_STATUS_ALLOCATING
+            data['status'] = n_const.ROUTER_STATUS_ALLOCATING
 
         with context.session.begin(subtransactions=True):
             router_db = super(L3_HA_NAT_db_mixin, self)._update_router_db(
@@ -546,10 +578,14 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
             self._set_vr_id(context, router_db, ha_network)
         else:
             self._delete_ha_interfaces(context, router_db.id)
+            # always attempt to cleanup the network as the router is
+            # deleted. the core plugin will stop us if its in use
+            self.safe_delete_ha_network(context, ha_network,
+                                        router_db.tenant_id)
 
         self.schedule_router(context, router_id)
         router_db = super(L3_HA_NAT_db_mixin, self)._update_router_db(
-            context, router_id, {'status': constants.ROUTER_STATUS_ACTIVE})
+            context, router_id, {'status': n_const.ROUTER_STATUS_ACTIVE})
         self._notify_ha_interfaces_updated(context, router_db.id,
                                            schedule_routers=False)
 
@@ -559,17 +595,30 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
         admin_ctx = context.elevated()
         self._core_plugin.delete_network(admin_ctx, net.network_id)
 
-    def _ha_routers_present(self, context, tenant_id):
-        ha = True
-        routers = context.session.query(l3_db.Router).filter(
-            l3_db.Router.tenant_id == tenant_id).subquery()
-        ha_routers = context.session.query(
-            l3_attrs_db.RouterExtraAttributes).join(
-            routers,
-            l3_attrs_db.RouterExtraAttributes.router_id == routers.c.id
-        ).filter(l3_attrs_db.RouterExtraAttributes.ha == ha).first()
-        return ha_routers is not None
+    def safe_delete_ha_network(self, context, ha_network, tenant_id):
+        try:
+            # reference the attr inside the try block before we attempt
+            # to delete the network and potentially invalidate the
+            # relationship
+            net_id = ha_network.network_id
+            self._delete_ha_network(context, ha_network)
+        except (n_exc.NetworkNotFound,
+                orm.exc.ObjectDeletedError):
+            LOG.debug(
+                "HA network for tenant %s was already deleted.", tenant_id)
+        except sa.exc.InvalidRequestError:
+            LOG.info(_LI("HA network %s can not be deleted."), net_id)
+        except n_exc.NetworkInUse:
+            # network is still in use, this is normal so we don't
+            # log anything
+            pass
+        else:
+            LOG.info(_LI("HA network %(network)s was deleted as "
+                         "no HA routers are present in tenant "
+                         "%(tenant)s."),
+                     {'network': net_id, 'tenant': tenant_id})
 
+    @db_api.retry_if_session_inactive()
     def delete_router(self, context, id):
         router_db = self._get_router(context, id)
         super(L3_HA_NAT_db_mixin, self).delete_router(context, id)
@@ -580,32 +629,11 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
             if ha_network:
                 self._delete_vr_id_allocation(
                     context, ha_network, router_db.extra_attributes.ha_vr_id)
-                self._delete_ha_interfaces(context, router_db.id)
 
-                # In case that create HA router failed because of the failure
-                # in HA network creation. So here put this deleting HA network
-                # procedure under 'if ha_network' block.
-                if not self._ha_routers_present(context,
-                                                router_db.tenant_id):
-                    try:
-                        self._delete_ha_network(context, ha_network)
-                    except (n_exc.NetworkNotFound,
-                            orm.exc.ObjectDeletedError):
-                        LOG.debug(
-                            "HA network for tenant %s was already deleted.",
-                            router_db.tenant_id)
-                    except sa.exc.InvalidRequestError:
-                        LOG.info(_LI("HA network %s can not be deleted."),
-                                 ha_network.network_id)
-                    except n_exc.NetworkInUse:
-                        LOG.debug("HA network %s is still in use.",
-                                  ha_network.network_id)
-                    else:
-                        LOG.info(_LI("HA network %(network)s was deleted as "
-                                     "no HA routers are present in tenant "
-                                     "%(tenant)s."),
-                                 {'network': ha_network.network_id,
-                                  'tenant': router_db.tenant_id})
+                # always attempt to cleanup the network as the router is
+                # deleted. the core plugin will stop us if its in use
+                self.safe_delete_ha_network(context, ha_network,
+                                            router_db.tenant_id)
 
     def _unbind_ha_router(self, context, router_id):
         for agent in self.get_l3_agents_hosting_routers(context, [router_id]):
@@ -643,11 +671,12 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
             bindings = self.get_ha_router_port_bindings(context, [router_id])
             dead_agents = [
                 binding.agent for binding in bindings
-                if binding.state == constants.HA_ROUTER_STATE_ACTIVE and
-                not binding.agent.is_active]
+                if binding.state == n_const.HA_ROUTER_STATE_ACTIVE and
+                not (binding.agent.is_active and binding.agent.admin_state_up)]
+
             for dead_agent in dead_agents:
                 self.update_routers_states(
-                    context, {router_id: constants.HA_ROUTER_STATE_STANDBY},
+                    context, {router_id: n_const.HA_ROUTER_STATE_STANDBY},
                     dead_agent.host)
 
         if dead_agents:
@@ -673,29 +702,42 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
         # and l2pop would not work correctly.
         return next(
             (agent.host for agent, state in bindings
-             if state == constants.HA_ROUTER_STATE_ACTIVE),
+             if state == n_const.HA_ROUTER_STATE_ACTIVE),
             None)
 
     @log_helpers.log_method_call
-    def _process_sync_ha_data(self, context, routers, host):
+    def _process_sync_ha_data(self, context, routers, host, agent_mode):
         routers_dict = dict((router['id'], router) for router in routers)
 
         bindings = self.get_ha_router_port_bindings(context,
                                                     routers_dict.keys(),
                                                     host)
         for binding in bindings:
-            port_dict = self._core_plugin._make_port_dict(binding.port)
+            port = binding.port
+            if not port:
+                # Filter the HA router has no ha port here
+                LOG.info(_LI("HA router %s is missing HA router port "
+                             "bindings. Skipping it."),
+                         binding.router_id)
+                routers_dict.pop(binding.router_id)
+                continue
+            port_dict = self._core_plugin._make_port_dict(port)
 
             router = routers_dict.get(binding.router_id)
             router[constants.HA_INTERFACE_KEY] = port_dict
-            router[constants.HA_ROUTER_STATE_KEY] = binding.state
+            router[n_const.HA_ROUTER_STATE_KEY] = binding.state
 
         for router in routers_dict.values():
             interface = router.get(constants.HA_INTERFACE_KEY)
             if interface:
                 self._populate_mtu_and_subnets_for_ports(context, [interface])
 
-        return list(routers_dict.values())
+        # If this is a DVR+HA router, but the agent is question is in 'dvr'
+        # mode (as opposed to 'dvr_snat'), then we want to always return it
+        # even though it's missing the '_ha_interface' key.
+        return [r for r in list(routers_dict.values())
+                if (agent_mode == constants.L3_AGENT_MODE_DVR or
+                    not r.get('ha') or r.get(constants.HA_INTERFACE_KEY))]
 
     @log_helpers.log_method_call
     def get_ha_sync_data_for_host(self, context, host, agent,
@@ -711,7 +753,7 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
         else:
             sync_data = super(L3_HA_NAT_db_mixin, self).get_sync_data(context,
                                                             router_ids, active)
-        return self._process_sync_ha_data(context, sync_data, host)
+        return self._process_sync_ha_data(context, sync_data, host, agent_mode)
 
     @classmethod
     def _set_router_states(cls, context, bindings, states):
@@ -723,6 +765,7 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
                 # Take concurrently deleted routers in to account
                 pass
 
+    @db_api.retry_if_session_inactive()
     def update_routers_states(self, context, states, host):
         """Receive dict of router ID to state and update them all."""
 
@@ -733,13 +776,13 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
 
     def _update_router_port_bindings(self, context, states, host):
         admin_ctx = context.elevated()
-        device_filter = {'device_id': states.keys(),
+        device_filter = {'device_id': list(states.keys()),
                          'device_owner':
-                         [constants.DEVICE_OWNER_ROUTER_INTF,
+                         [constants.DEVICE_OWNER_HA_REPLICATED_INT,
                           constants.DEVICE_OWNER_ROUTER_SNAT]}
         ports = self._core_plugin.get_ports(admin_ctx, filters=device_filter)
         active_ports = (port for port in ports
-            if states[port['device_id']] == constants.HA_ROUTER_STATE_ACTIVE)
+            if states[port['device_id']] == n_const.HA_ROUTER_STATE_ACTIVE)
 
         for port in active_ports:
             port[portbindings.HOST_ID] = host
@@ -750,3 +793,30 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
                     n_exc.PortNotFound):
                 # Take concurrently deleted interfaces in to account
                 pass
+
+
+def is_ha_router(router):
+    """Return True if router to be handled is ha."""
+    try:
+        # See if router is a DB object first
+        requested_router_type = router.extra_attributes.ha
+    except AttributeError:
+        # if not, try to see if it is a request body
+        requested_router_type = router.get('ha')
+    if validators.is_attr_set(requested_router_type):
+        return requested_router_type
+    return cfg.CONF.l3_ha
+
+
+def is_ha_router_port(device_owner, router_id):
+    session = db_api.get_session()
+    if device_owner == constants.DEVICE_OWNER_HA_REPLICATED_INT:
+        return True
+    elif device_owner == constants.DEVICE_OWNER_ROUTER_SNAT:
+        query = session.query(l3_attrs_db.RouterExtraAttributes)
+        query = query.filter_by(ha=True)
+        query = query.filter(l3_attrs_db.RouterExtraAttributes.router_id ==
+                             router_id)
+        return bool(query.limit(1).count())
+    else:
+        return False
