@@ -13,59 +13,28 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from neutron_lib import constants
+from neutron_lib.db import model_base
 import sqlalchemy as sa
 from sqlalchemy import orm
 from sqlalchemy import sql
 
 from neutron.api.v2 import attributes as attr
-from neutron.common import constants
-from neutron.db import agentschedulers_db as agt
-from neutron.db import model_base
+from neutron.common import _deprecate
+from neutron.db.network_dhcp_agent_binding import models as ndab_model
 from neutron.db import rbac_db_models
+from neutron.db import standard_attr
 
 
 # NOTE(kevinbenton): these are here for external projects that expect them
 # to be found in this module.
-HasTenant = model_base.HasTenant
-HasId = model_base.HasId
-HasStatusDescription = model_base.HasStatusDescription
+_deprecate._moved_global('HasTenant', new_name='HasProject',
+                         new_module=model_base)
+_deprecate._moved_global('HasId', new_module=model_base)
+_deprecate._moved_global('HasStatusDescription', new_module=model_base)
 
 
-class IPAvailabilityRange(model_base.BASEV2):
-    """Internal representation of available IPs for Neutron subnets.
-
-    Allocation - first entry from the range will be allocated.
-    If the first entry is equal to the last entry then this row
-    will be deleted.
-    Recycling ips involves reading the IPAllocationPool and IPAllocation tables
-    and inserting ranges representing available ips.  This happens after the
-    final allocation is pulled from this table and a new ip allocation is
-    requested.  Any contiguous ranges of available ips will be inserted as a
-    single range.
-    """
-
-    allocation_pool_id = sa.Column(sa.String(36),
-                                   sa.ForeignKey('ipallocationpools.id',
-                                                 ondelete="CASCADE"),
-                                   nullable=False,
-                                   primary_key=True)
-    first_ip = sa.Column(sa.String(64), nullable=False, primary_key=True)
-    last_ip = sa.Column(sa.String(64), nullable=False, primary_key=True)
-    __table_args__ = (
-        sa.UniqueConstraint(
-            first_ip, allocation_pool_id,
-            name='uniq_ipavailabilityranges0first_ip0allocation_pool_id'),
-        sa.UniqueConstraint(
-            last_ip, allocation_pool_id,
-            name='uniq_ipavailabilityranges0last_ip0allocation_pool_id'),
-        model_base.BASEV2.__table_args__
-    )
-
-    def __repr__(self):
-        return "%s - %s" % (self.first_ip, self.last_ip)
-
-
-class IPAllocationPool(model_base.BASEV2, HasId):
+class IPAllocationPool(model_base.BASEV2, model_base.HasId):
     """Representation of an allocation pool in a Neutron subnet."""
 
     subnet_id = sa.Column(sa.String(36), sa.ForeignKey('subnets.id',
@@ -73,10 +42,6 @@ class IPAllocationPool(model_base.BASEV2, HasId):
                           nullable=True)
     first_ip = sa.Column(sa.String(64), nullable=False)
     last_ip = sa.Column(sa.String(64), nullable=False)
-    available_ranges = orm.relationship(IPAvailabilityRange,
-                                        backref='ipallocationpool',
-                                        lazy="select",
-                                        cascade='all, delete-orphan')
 
     def __repr__(self):
         return "%s - %s" % (self.first_ip, self.last_ip)
@@ -96,6 +61,7 @@ class IPAllocation(model_base.BASEV2):
     network_id = sa.Column(sa.String(36), sa.ForeignKey("networks.id",
                                                         ondelete="CASCADE"),
                            nullable=False, primary_key=True)
+    revises_on_change = ('port', )
 
 
 class Route(object):
@@ -113,8 +79,8 @@ class SubnetRoute(model_base.BASEV2, Route):
                           primary_key=True)
 
 
-class Port(model_base.HasStandardAttributes, model_base.BASEV2,
-           HasId, HasTenant):
+class Port(standard_attr.HasStandardAttributes, model_base.BASEV2,
+           model_base.HasId, model_base.HasProject):
     """Represents a port on a Neutron v2 network."""
 
     name = sa.Column(sa.String(attr.NAME_MAX_LEN))
@@ -129,22 +95,24 @@ class Port(model_base.HasStandardAttributes, model_base.BASEV2,
     device_id = sa.Column(sa.String(attr.DEVICE_ID_MAX_LEN), nullable=False)
     device_owner = sa.Column(sa.String(attr.DEVICE_OWNER_MAX_LEN),
                              nullable=False)
-    dns_name = sa.Column(sa.String(255), nullable=True)
+    ip_allocation = sa.Column(sa.String(16))
+
     __table_args__ = (
         sa.Index(
             'ix_ports_network_id_mac_address', 'network_id', 'mac_address'),
         sa.Index(
             'ix_ports_network_id_device_owner', 'network_id', 'device_owner'),
+        sa.Index('ix_ports_device_id', 'device_id'),
         sa.UniqueConstraint(
             network_id, mac_address,
             name='uniq_ports0network_id0mac_address'),
         model_base.BASEV2.__table_args__
     )
+    api_collections = [attr.PORTS]
 
     def __init__(self, id=None, tenant_id=None, name=None, network_id=None,
                  mac_address=None, admin_state_up=None, status=None,
-                 device_id=None, device_owner=None, fixed_ips=None,
-                 dns_name=None, **kwargs):
+                 device_id=None, device_owner=None, fixed_ips=None, **kwargs):
         super(Port, self).__init__(**kwargs)
         self.id = id
         self.tenant_id = tenant_id
@@ -154,7 +122,6 @@ class Port(model_base.HasStandardAttributes, model_base.BASEV2,
         self.admin_state_up = admin_state_up
         self.device_owner = device_owner
         self.device_id = device_id
-        self.dns_name = dns_name
         # Since this is a relationship only set it if one is passed in.
         if fixed_ips:
             self.fixed_ips = fixed_ips
@@ -174,8 +141,8 @@ class DNSNameServer(model_base.BASEV2):
     order = sa.Column(sa.Integer, nullable=False, server_default='0')
 
 
-class Subnet(model_base.HasStandardAttributes, model_base.BASEV2,
-             HasId, HasTenant):
+class Subnet(standard_attr.HasStandardAttributes, model_base.BASEV2,
+             model_base.HasId, model_base.HasProject):
     """Represents a neutron subnet.
 
     When a subnet is created the first and last entries will be created. These
@@ -184,6 +151,8 @@ class Subnet(model_base.HasStandardAttributes, model_base.BASEV2,
 
     name = sa.Column(sa.String(attr.NAME_MAX_LEN))
     network_id = sa.Column(sa.String(36), sa.ForeignKey('networks.id'))
+    # Added by the segments service plugin
+    segment_id = sa.Column(sa.String(36), sa.ForeignKey('networksegments.id'))
     subnetpool_id = sa.Column(sa.String(36), index=True)
     # NOTE: Explicitly specify join conditions for the relationship because
     # subnetpool_id in subnet might be 'prefix_delegation' when the IPv6 Prefix
@@ -195,6 +164,7 @@ class Subnet(model_base.HasStandardAttributes, model_base.BASEV2,
     ip_version = sa.Column(sa.Integer, nullable=False)
     cidr = sa.Column(sa.String(64), nullable=False)
     gateway_ip = sa.Column(sa.String(64))
+    revises_on_change = ('networks', )
     allocation_pools = orm.relationship(IPAllocationPool,
                                         backref='subnet',
                                         lazy="joined",
@@ -212,17 +182,20 @@ class Subnet(model_base.HasStandardAttributes, model_base.BASEV2,
     ipv6_ra_mode = sa.Column(sa.Enum(constants.IPV6_SLAAC,
                                      constants.DHCPV6_STATEFUL,
                                      constants.DHCPV6_STATELESS,
-                                     name='ipv6_ra_modes'), nullable=True)
+                                     name='ipv6_ra_modes'),
+                             nullable=True)
     ipv6_address_mode = sa.Column(sa.Enum(constants.IPV6_SLAAC,
-                                  constants.DHCPV6_STATEFUL,
-                                  constants.DHCPV6_STATELESS,
-                                  name='ipv6_address_modes'), nullable=True)
+                                          constants.DHCPV6_STATEFUL,
+                                          constants.DHCPV6_STATELESS,
+                                          name='ipv6_address_modes'),
+                                  nullable=True)
     # subnets don't have their own rbac_entries, they just inherit from
     # the network rbac entries
     rbac_entries = orm.relationship(
         rbac_db_models.NetworkRBAC, lazy='joined', uselist=True,
         foreign_keys='Subnet.network_id',
         primaryjoin='Subnet.network_id==NetworkRBAC.object_id')
+    api_collections = [attr.SUBNETS]
 
 
 class SubnetPoolPrefix(model_base.BASEV2):
@@ -239,8 +212,8 @@ class SubnetPoolPrefix(model_base.BASEV2):
                               primary_key=True)
 
 
-class SubnetPool(model_base.HasStandardAttributes, model_base.BASEV2,
-                 HasId, HasTenant):
+class SubnetPool(standard_attr.HasStandardAttributes, model_base.BASEV2,
+                 model_base.HasId, model_base.HasProject):
     """Represents a neutron subnet pool.
     """
 
@@ -259,10 +232,11 @@ class SubnetPool(model_base.HasStandardAttributes, model_base.BASEV2,
                                 backref='subnetpools',
                                 cascade='all, delete, delete-orphan',
                                 lazy='joined')
+    api_collections = [attr.SUBNETPOOLS]
 
 
-class Network(model_base.HasStandardAttributes, model_base.BASEV2,
-              HasId, HasTenant):
+class Network(standard_attr.HasStandardAttributes, model_base.BASEV2,
+              model_base.HasId, model_base.HasProject):
     """Represents a v2 neutron network."""
 
     name = sa.Column(sa.String(attr.NAME_MAX_LEN))
@@ -272,7 +246,6 @@ class Network(model_base.HasStandardAttributes, model_base.BASEV2,
         lazy="joined")
     status = sa.Column(sa.String(16))
     admin_state_up = sa.Column(sa.Boolean)
-    mtu = sa.Column(sa.Integer, nullable=True)
     vlan_transparent = sa.Column(sa.Boolean, nullable=True)
     rbac_entries = orm.relationship(rbac_db_models.NetworkRBAC,
                                     backref='network', lazy='joined',
@@ -280,4 +253,8 @@ class Network(model_base.HasStandardAttributes, model_base.BASEV2,
     availability_zone_hints = sa.Column(sa.String(255))
     dhcp_agents = orm.relationship(
         'Agent', lazy='joined', viewonly=True,
-        secondary=agt.NetworkDhcpAgentBinding.__table__)
+        secondary=ndab_model.NetworkDhcpAgentBinding.__table__)
+    api_collections = [attr.NETWORKS]
+
+
+_deprecate._MovedGlobals()
