@@ -18,18 +18,17 @@ import random
 
 import eventlet
 import mock
+from neutron_lib import constants as n_const
 from oslo_config import cfg
 from oslo_utils import uuidutils
 
 from neutron.agent.common import config as agent_config
 from neutron.agent.common import ovs_lib
-from neutron.agent.l2.extensions import manager as ext_manager
+from neutron.agent.l2 import l2_agent_extensions_manager as ext_manager
 from neutron.agent.linux import interface
 from neutron.agent.linux import polling
-from neutron.agent.linux import utils as agent_utils
-from neutron.common import config as common_config
-from neutron.common import constants as n_const
 from neutron.common import utils
+from neutron.conf import common as common_config
 from neutron.plugins.common import constants as p_const
 from neutron.plugins.ml2.drivers.openvswitch.agent.common import config \
     as ovs_config
@@ -54,12 +53,12 @@ class OVSAgentTestFramework(base.BaseOVSLinuxTestCase):
                      'ovs_neutron_agent.OVSPluginApi')
         mock.patch(agent_rpc).start()
         mock.patch('neutron.agent.rpc.PluginReportStateAPI').start()
-        self.br_int = base.get_rand_name(n_const.DEVICE_NAME_MAX_LEN,
-                                         prefix='br-int')
-        self.br_tun = base.get_rand_name(n_const.DEVICE_NAME_MAX_LEN,
-                                         prefix='br-tun')
-        self.br_phys = base.get_rand_name(n_const.DEVICE_NAME_MAX_LEN,
-                                          prefix='br-phys')
+        self.br_int = utils.get_rand_name(n_const.DEVICE_NAME_MAX_LEN,
+                                          prefix='br-int')
+        self.br_tun = utils.get_rand_name(n_const.DEVICE_NAME_MAX_LEN,
+                                          prefix='br-tun')
+        self.br_phys = utils.get_rand_name(n_const.DEVICE_NAME_MAX_LEN,
+                                           prefix='br-phys')
         patch_name_len = n_const.DEVICE_NAME_MAX_LEN - len("-patch-tun")
         self.patch_tun = "%s-patch-tun" % self.br_int[patch_name_len:]
         self.patch_int = "%s-patch-int" % self.br_tun[patch_name_len:]
@@ -99,7 +98,8 @@ class OVSAgentTestFramework(base.BaseOVSLinuxTestCase):
             'br_tun': br_tun.OVSTunnelBridge
         }
 
-    def create_agent(self, create_tunnels=True, ancillary_bridge=None):
+    def create_agent(self, create_tunnels=True, ancillary_bridge=None,
+                     local_ip='192.168.10.1'):
         if create_tunnels:
             tunnel_types = [p_const.TYPE_VXLAN]
         else:
@@ -108,7 +108,7 @@ class OVSAgentTestFramework(base.BaseOVSLinuxTestCase):
         self.config.set_override('tunnel_types', tunnel_types, "AGENT")
         self.config.set_override('polling_interval', 1, "AGENT")
         self.config.set_override('prevent_arp_spoofing', False, "AGENT")
-        self.config.set_override('local_ip', '192.168.10.1', "OVS")
+        self.config.set_override('local_ip', local_ip, "OVS")
         self.config.set_override('bridge_mappings', bridge_mappings, "OVS")
         # Physical bridges should be created prior to running
         self._bridge_classes()['br_phys'](self.br_phys).create()
@@ -145,6 +145,10 @@ class OVSAgentTestFramework(base.BaseOVSLinuxTestCase):
             return {'added': filtered_ports, 'removed': events['removed']}
         polling_manager.get_events = mock.Mock(side_effect=filter_events)
 
+    def stop_agent(self, agent, rpc_loop_thread):
+        agent.run_daemon_loop = False
+        rpc_loop_thread.wait()
+
     def start_agent(self, agent, ports=None, unplug_ports=None):
         if unplug_ports is None:
             unplug_ports = []
@@ -155,17 +159,14 @@ class OVSAgentTestFramework(base.BaseOVSLinuxTestCase):
         self._mock_get_events(agent, polling_manager, ports)
         self.addCleanup(polling_manager.stop)
         polling_manager.start()
-        agent_utils.wait_until_true(
+        utils.wait_until_true(
             polling_manager._monitor.is_active)
         agent.check_ovs_status = mock.Mock(
             return_value=constants.OVS_NORMAL)
-        t = eventlet.spawn(agent.rpc_loop, polling_manager)
+        self.agent_thread = eventlet.spawn(agent.rpc_loop,
+                                           polling_manager)
 
-        def stop_agent(agent, rpc_loop_thread):
-            agent.run_daemon_loop = False
-            rpc_loop_thread.wait()
-
-        self.addCleanup(stop_agent, agent, t)
+        self.addCleanup(self.stop_agent, agent, self.agent_thread)
         return polling_manager
 
     def _create_test_port_dict(self):
@@ -177,7 +178,7 @@ class OVSAgentTestFramework(base.BaseOVSLinuxTestCase):
                          random.randint(3, 254),
                          random.randint(3, 254),
                          random.randint(3, 254))}],
-                'vif_name': base.get_rand_name(
+                'vif_name': utils.get_rand_name(
                     self.driver.DEV_NAME_LEN, self.driver.DEV_NAME_PREFIX)}
 
     def _create_test_network_dict(self):
@@ -212,7 +213,7 @@ class OVSAgentTestFramework(base.BaseOVSLinuxTestCase):
                'physical_network': network.get('physical_network', 'physnet'),
                'segmentation_id': network.get('segmentation_id', 1),
                'fixed_ips': port['fixed_ips'],
-               'device_owner': 'compute',
+               'device_owner': n_const.DEVICE_OWNER_COMPUTE_PREFIX,
                'port_security_enabled': True,
                'security_groups': ['default'],
                'admin_state_up': True}
@@ -227,9 +228,9 @@ class OVSAgentTestFramework(base.BaseOVSLinuxTestCase):
             return agent.int_br.db_get_val(
                 'Interface', port, 'options', check_error=True)
 
-        agent_utils.wait_until_true(
+        utils.wait_until_true(
             lambda: get_peer(self.patch_int) == {'peer': self.patch_tun})
-        agent_utils.wait_until_true(
+        utils.wait_until_true(
             lambda: get_peer(self.patch_tun) == {'peer': self.patch_int})
 
     def assert_bridge_ports(self):
@@ -360,7 +361,7 @@ class OVSAgentTestFramework(base.BaseOVSLinuxTestCase):
 
     def wait_until_ports_state(self, ports, up, timeout=60):
         port_ids = [p['id'] for p in ports]
-        agent_utils.wait_until_true(
+        utils.wait_until_true(
             lambda: self._expected_plugin_rpc_call(
                 self.agent.plugin_rpc.update_device_list, port_ids, up),
             timeout=timeout)
