@@ -13,14 +13,17 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import copy
 import functools
 
 import mock
 import netaddr
+from neutron_lib import constants
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import uuidutils
 import testtools
+import textwrap
 
 from neutron.agent.common import config as agent_config
 from neutron.agent.common import ovs_lib
@@ -28,10 +31,9 @@ from neutron.agent.l3 import agent as neutron_l3_agent
 from neutron.agent import l3_agent as l3_agent_main
 from neutron.agent.linux import external_process
 from neutron.agent.linux import ip_lib
-from neutron.agent.linux import utils
-from neutron.common import config as common_config
-from neutron.common import constants as l3_constants
+from neutron.agent.linux import keepalived
 from neutron.common import utils as common_utils
+from neutron.conf import common as common_config
 from neutron.tests.common import l3_test_common
 from neutron.tests.common import net_helpers
 from neutron.tests.functional import base
@@ -45,6 +47,8 @@ def get_ovs_bridge(br_name):
 
 
 class L3AgentTestFramework(base.BaseSudoTestCase):
+    INTERFACE_DRIVER = 'neutron.agent.linux.interface.OVSInterfaceDriver'
+
     def setUp(self):
         super(L3AgentTestFramework, self).setUp()
         self.mock_plugin_api = mock.patch(
@@ -65,9 +69,7 @@ class L3AgentTestFramework(base.BaseSudoTestCase):
     def _configure_agent(self, host, agent_mode='dvr_snat'):
         conf = self._get_config_opts()
         l3_agent_main.register_opts(conf)
-        conf.set_override(
-            'interface_driver',
-            'neutron.agent.linux.interface.OVSInterfaceDriver')
+        conf.set_override('interface_driver', self.INTERFACE_DRIVER)
 
         br_int = self.useFixture(net_helpers.OVSBridgeFixture()).bridge
         br_ex = self.useFixture(net_helpers.OVSBridgeFixture()).bridge
@@ -123,13 +125,13 @@ class L3AgentTestFramework(base.BaseSudoTestCase):
         router_info = self.generate_router_info(enable_ha=ha)
         router = self.manage_router(self.agent, router_info)
 
-        port = net_helpers.get_free_namespace_port(l3_constants.PROTO_NAME_TCP,
-                                                   router.ns_name)
+        port = net_helpers.get_free_namespace_port(
+            constants.PROTO_NAME_TCP, router.ns_name)
         client_address = '19.4.4.3'
         server_address = '35.4.0.4'
 
         def clean_fips(router):
-            router.router[l3_constants.FLOATINGIP_KEY] = []
+            router.router[constants.FLOATINGIP_KEY] = []
 
         clean_fips(router)
         self._add_fip(router, client_address, fixed_address=server_address)
@@ -148,7 +150,7 @@ class L3AgentTestFramework(base.BaseSudoTestCase):
                 n, len([line for line in out.strip().split('\n') if line]))
 
         if ha:
-            utils.wait_until_true(lambda: router.ha_state == 'master')
+            common_utils.wait_until_true(lambda: router.ha_state == 'master')
 
         with self.assert_max_execution_time(100):
             assert_num_of_conntrack_rules(0)
@@ -170,8 +172,8 @@ class L3AgentTestFramework(base.BaseSudoTestCase):
 
         # Assert that every defined FIP is updated via RPC
         expected_fips = set([
-            (fip['id'], l3_constants.FLOATINGIP_STATUS_ACTIVE) for fip in
-            router.router[l3_constants.FLOATINGIP_KEY]])
+            (fip['id'], constants.FLOATINGIP_STATUS_ACTIVE) for fip in
+            router.router[constants.FLOATINGIP_KEY]])
         call = [args[0] for args in rpc.call_args_list][0]
         actual_fips = set(
             [(fip_id, status) for fip_id, status in call[2].items()])
@@ -188,7 +190,7 @@ class L3AgentTestFramework(base.BaseSudoTestCase):
         def ha_router_dev_name_getter(not_used):
             return router.get_ha_device_name()
         self.assertTrue(self.device_exists_with_ips_and_mac(
-            router.router[l3_constants.HA_INTERFACE_KEY],
+            router.router[constants.HA_INTERFACE_KEY],
             ha_router_dev_name_getter, router.ns_name))
 
     def _assert_gateway(self, router, v6_ext_gw_with_sub=True):
@@ -209,16 +211,26 @@ class L3AgentTestFramework(base.BaseSudoTestCase):
             external_port, router.get_external_device_name,
             router.ns_name))
 
+    def _assert_ipv6_accept_ra(self, router):
+        external_port = router.get_ex_gw_port()
+        external_device_name = router.get_external_device_name(
+            external_port['id'])
+        ip_wrapper = ip_lib.IPWrapper(namespace=router.ns_name)
+        ra_state = ip_wrapper.netns.execute(['sysctl', '-b',
+            'net.ipv6.conf.%s.accept_ra' % external_device_name])
+        self.assertEqual('2', ra_state)
+
     def _router_lifecycle(self, enable_ha, ip_version=4,
-                          dual_stack=False, v6_ext_gw_with_sub=True):
-        router_info = self.generate_router_info(enable_ha, ip_version,
-                                                dual_stack=dual_stack,
-                                                v6_ext_gw_with_sub=(
-                                                    v6_ext_gw_with_sub))
+                          dual_stack=False, v6_ext_gw_with_sub=True,
+                          router_info=None):
+        router_info = router_info or self.generate_router_info(
+            enable_ha, ip_version, dual_stack=dual_stack,
+            v6_ext_gw_with_sub=(v6_ext_gw_with_sub))
+        return_copy = copy.deepcopy(router_info)
         router = self.manage_router(self.agent, router_info)
 
         # Add multiple-IPv6-prefix internal router port
-        slaac = l3_constants.IPV6_SLAAC
+        slaac = constants.IPV6_SLAAC
         slaac_mode = {'ra_mode': slaac, 'address_mode': slaac}
         subnet_modes = [slaac_mode] * 2
         self._add_internal_interface_by_subnet(router.router,
@@ -232,22 +244,22 @@ class L3AgentTestFramework(base.BaseSudoTestCase):
             interface_name = router.get_external_device_name(port['id'])
             self._assert_no_ip_addresses_on_interface(router.ns_name,
                                                       interface_name)
-            utils.wait_until_true(lambda: router.ha_state == 'master')
+            common_utils.wait_until_true(lambda: router.ha_state == 'master')
 
             # Keepalived notifies of a state transition when it starts,
             # not when it ends. Thus, we have to wait until keepalived finishes
             # configuring everything. We verify this by waiting until the last
             # device has an IP address.
-            device = router.router[l3_constants.INTERFACE_KEY][-1]
+            device = router.router[constants.INTERFACE_KEY][-1]
             device_exists = functools.partial(
                 self.device_exists_with_ips_and_mac,
                 device,
                 router.get_internal_device_name,
                 router.ns_name)
-            utils.wait_until_true(device_exists)
+            common_utils.wait_until_true(device_exists)
 
         self.assertTrue(self._namespace_exists(router.ns_name))
-        utils.wait_until_true(
+        common_utils.wait_until_true(
             lambda: self._metadata_proxy_exists(self.agent.conf, router))
         self._assert_internal_devices(router)
         self._assert_external_device(router)
@@ -270,13 +282,7 @@ class L3AgentTestFramework(base.BaseSudoTestCase):
         # when IPv6 is enabled and no IPv6 gateway is configured.
         if router.use_ipv6 and not v6_ext_gw_with_sub:
             if not self.agent.conf.ipv6_gateway:
-                external_port = router.get_ex_gw_port()
-                external_device_name = router.get_external_device_name(
-                    external_port['id'])
-                ip_wrapper = ip_lib.IPWrapper(namespace=router.ns_name)
-                ra_state = ip_wrapper.netns.execute(['sysctl', '-b',
-                    'net.ipv6.conf.%s.accept_ra' % external_device_name])
-                self.assertEqual('2', ra_state)
+                self._assert_ipv6_accept_ra(router)
 
         if enable_ha:
             self._assert_ha_device(router)
@@ -288,6 +294,7 @@ class L3AgentTestFramework(base.BaseSudoTestCase):
         self._assert_router_does_not_exist(router)
         if enable_ha:
             self.assertFalse(router.keepalived_manager.get_process().active)
+        return return_copy
 
     def manage_router(self, agent, router):
         self.addCleanup(agent._safe_router_removed, router['id'])
@@ -305,7 +312,7 @@ class L3AgentTestFramework(base.BaseSudoTestCase):
                'fixed_ip_address': fixed_address,
                'host': host,
                'fixed_ip_address_scope': fixed_ip_address_scope}
-        router.router[l3_constants.FLOATINGIP_KEY].append(fip)
+        router.router[constants.FLOATINGIP_KEY].append(fip)
 
     def _add_internal_interface_by_subnet(self, router, count=1,
                                           ip_version=4,
@@ -346,10 +353,10 @@ class L3AgentTestFramework(base.BaseSudoTestCase):
         ha_device_name = router.get_ha_device_name()
         external_port = router.get_ex_gw_port()
         ex_port_ipv6 = ip_lib.get_ipv6_lladdr(external_port['mac_address'])
-        external_device_name = router.get_external_device_name(
+        ex_device_name = router.get_external_device_name(
             external_port['id'])
         external_device_cidr = self._port_first_ip_cidr(external_port)
-        internal_port = router.router[l3_constants.INTERFACE_KEY][0]
+        internal_port = router.router[constants.INTERFACE_KEY][0]
         int_port_ipv6 = ip_lib.get_ipv6_lladdr(internal_port['mac_address'])
         internal_device_name = router.get_internal_device_name(
             internal_port['id'])
@@ -358,35 +365,42 @@ class L3AgentTestFramework(base.BaseSudoTestCase):
             router.get_floating_ips()[0]['floating_ip_address'])
         default_gateway_ip = external_port['subnets'][0].get('gateway_ip')
         extra_subnet_cidr = external_port['extra_subnets'][0].get('cidr')
-        return """vrrp_instance VR_1 {
-    state BACKUP
-    interface %(ha_device_name)s
-    virtual_router_id 1
-    priority 50
-    garp_master_delay 60
-    nopreempt
-    advert_int 2
-    track_interface {
-        %(ha_device_name)s
-    }
-    virtual_ipaddress {
-        169.254.0.1/24 dev %(ha_device_name)s
-    }
-    virtual_ipaddress_excluded {
-        %(floating_ip_cidr)s dev %(external_device_name)s
-        %(external_device_cidr)s dev %(external_device_name)s
-        %(internal_device_cidr)s dev %(internal_device_name)s
-        %(ex_port_ipv6)s dev %(external_device_name)s scope link
-        %(int_port_ipv6)s dev %(internal_device_name)s scope link
-    }
-    virtual_routes {
-        0.0.0.0/0 via %(default_gateway_ip)s dev %(external_device_name)s
-        8.8.8.0/24 via 19.4.4.4
-        %(extra_subnet_cidr)s dev %(external_device_name)s scope link
-    }
-}""" % {
+        return textwrap.dedent("""\
+            global_defs {
+                notification_email_from %(email_from)s
+                router_id %(router_id)s
+            }
+            vrrp_instance VR_1 {
+                state BACKUP
+                interface %(ha_device_name)s
+                virtual_router_id 1
+                priority 50
+                garp_master_delay 60
+                nopreempt
+                advert_int 2
+                track_interface {
+                    %(ha_device_name)s
+                }
+                virtual_ipaddress {
+                    169.254.0.1/24 dev %(ha_device_name)s
+                }
+                virtual_ipaddress_excluded {
+                    %(floating_ip_cidr)s dev %(ex_device_name)s
+                    %(external_device_cidr)s dev %(ex_device_name)s
+                    %(internal_device_cidr)s dev %(internal_device_name)s
+                    %(ex_port_ipv6)s dev %(ex_device_name)s scope link
+                    %(int_port_ipv6)s dev %(internal_device_name)s scope link
+                }
+                virtual_routes {
+                    0.0.0.0/0 via %(default_gateway_ip)s dev %(ex_device_name)s
+                    8.8.8.0/24 via 19.4.4.4
+                    %(extra_subnet_cidr)s dev %(ex_device_name)s scope link
+                }
+            }""") % {
+            'email_from': keepalived.KEEPALIVED_EMAIL_FROM,
+            'router_id': keepalived.KEEPALIVED_ROUTER_ID,
             'ha_device_name': ha_device_name,
-            'external_device_name': external_device_name,
+            'ex_device_name': ex_device_name,
             'external_device_cidr': external_device_cidr,
             'internal_device_name': internal_device_name,
             'internal_device_cidr': internal_device_cidr,
@@ -407,7 +421,7 @@ class L3AgentTestFramework(base.BaseSudoTestCase):
         # then the devices and iptable rules have also been deleted,
         # so there's no need to check that explicitly.
         self.assertFalse(self._namespace_exists(router.ns_name))
-        utils.wait_until_true(
+        common_utils.wait_until_true(
             lambda: not self._metadata_proxy_exists(self.agent.conf, router))
 
     def _assert_snat_chains(self, router):
@@ -439,7 +453,7 @@ class L3AgentTestFramework(base.BaseSudoTestCase):
                                        metadata_port_filter))
 
     def _assert_internal_devices(self, router):
-        internal_devices = router.router[l3_constants.INTERFACE_KEY]
+        internal_devices = router.router[constants.INTERFACE_KEY]
         self.assertTrue(len(internal_devices))
         for device in internal_devices:
             self.assertTrue(self.device_exists_with_ips_and_mac(
@@ -478,7 +492,7 @@ class L3AgentTestFramework(base.BaseSudoTestCase):
         assert_ovs_bridge_empty(self.agent.conf.external_network_bridge)
 
     def floating_ips_configured(self, router):
-        floating_ips = router.router[l3_constants.FLOATINGIP_KEY]
+        floating_ips = router.router[constants.FLOATINGIP_KEY]
         external_port = router.get_ex_gw_port()
         return len(floating_ips) and all(
             ip_lib.device_exists_with_ips_and_mac(
