@@ -19,9 +19,9 @@
 
 import copy
 import functools
-import random
 
 import netaddr
+from neutron_lib import constants
 from oslo_config import cfg
 from oslo_log import log as logging
 import testscenarios
@@ -29,11 +29,13 @@ import testscenarios
 from neutron.agent import firewall
 from neutron.agent.linux import iptables_firewall
 from neutron.agent.linux import openvswitch_firewall
-from neutron.agent import securitygroups_rpc as sg_cfg
 from neutron.cmd.sanity import checks
-from neutron.common import constants
+from neutron.conf.agent import securitygroups_rpc as security_config
 from neutron.tests.common import conn_testers
+from neutron.tests.common import helpers
+from neutron.tests.functional.agent.linux import base as linux_base
 from neutron.tests.functional import base
+from neutron.tests.functional import constants as test_constants
 
 LOG = logging.getLogger(__name__)
 
@@ -49,7 +51,6 @@ reverse_transport_protocol = {
     conn_testers.ConnectionTester.UDP: conn_testers.ConnectionTester.TCP}
 
 DEVICE_OWNER_COMPUTE = constants.DEVICE_OWNER_COMPUTE_PREFIX + 'fake'
-VLAN_COUNT = 4096
 
 
 def skip_if_firewall(firewall_name):
@@ -76,22 +77,24 @@ def _add_rule(sg_rules, base, port_range_min=None, port_range_max=None):
 class BaseFirewallTestCase(base.BaseSudoTestCase):
     FAKE_SECURITY_GROUP_ID = 'fake_sg_id'
     MAC_SPOOFED = "fa:16:3e:9a:2f:48"
-    scenarios = [('IptablesFirewallDriver without ipset',
-                  {'enable_ipset': False,
-                   'initialize': 'initialize_iptables',
-                   'firewall_name': 'iptables'}),
-                 ('IptablesFirewallDriver with ipset',
-                  {'enable_ipset': True,
-                   'initialize': 'initialize_iptables',
-                   'firewall_name': 'iptables'}),
-                 ('OVS Firewall Driver',
-                  {'initialize': 'initialize_ovs',
-                   'firewall_name': 'openvswitch'})]
+    scenarios_iptables = testscenarios.multiply_scenarios(
+        [('IptablesFirewallDriver', {'initialize': 'initialize_iptables',
+                                     'firewall_name': 'iptables'})],
+        [('with ipset', {'enable_ipset': True}),
+         ('without ipset', {'enable_ipset': False})])
+
+    scenarios_ovs_fw_interfaces = testscenarios.multiply_scenarios(
+        [('OVS Firewall Driver', {'initialize': 'initialize_ovs',
+                                  'firewall_name': 'openvswitch'})],
+        linux_base.BaseOVSLinuxTestCase.scenarios)
+
+    scenarios = scenarios_iptables + scenarios_ovs_fw_interfaces
+
     ip_cidr = None
-    vlan_range = set(range(VLAN_COUNT))
+    vlan_range = set(range(test_constants.VLAN_COUNT))
 
     def setUp(self):
-        cfg.CONF.register_opts(sg_cfg.security_group_opts, 'SECURITYGROUP')
+        security_config.register_securitygroups_opts()
         super(BaseFirewallTestCase, self).setUp()
         self.tester, self.firewall = getattr(self, self.initialize)()
         if self.firewall_name == "openvswitch":
@@ -115,12 +118,13 @@ class BaseFirewallTestCase(base.BaseSudoTestCase):
         return tester, firewall_drv
 
     def initialize_ovs(self):
+        self.config(group='OVS', ovsdb_interface=self.ovsdb_interface)
         # Tests for ovs requires kernel >= 4.3 and OVS >= 2.5
         if not checks.ovs_conntrack_supported():
             self.skipTest("Open vSwitch with conntrack is not installed "
                           "on this machine. To run tests for OVS/CT firewall,"
                           " please meet the requirements (kernel>=4.3, "
-                          "OVS>=2.5. More info at"
+                          "OVS>=2.5). More info at "
                           "https://github.com/openvswitch/ovs/blob/master/"
                           "FAQ.md")
         tester = self.useFixture(
@@ -129,18 +133,12 @@ class BaseFirewallTestCase(base.BaseSudoTestCase):
         return tester, firewall_drv
 
     def assign_vlan_to_peers(self):
-        vlan = self.get_not_used_vlan()
+        vlan = helpers.get_not_used_vlan(self.firewall.int_br.br,
+                                         self.vlan_range)
         LOG.debug("Using %d vlan tag for this test", vlan)
         self.tester.set_vm_tag(vlan)
         self.tester.set_peer_tag(vlan)
 
-    def get_not_used_vlan(self):
-        port_vlans = self.firewall.int_br.br.ovsdb.db_find(
-                'Port', ('tag', '!=', '[]'), columns=['tag']).execute()
-        used_vlan_tags = {val['tag'] for val in port_vlans}
-        available_vlans = self.vlan_range - used_vlan_tags
-        return random.choice(list(available_vlans))
-
     @staticmethod
     def _create_port_description(port_id, ip_addresses, mac_address, sg_ids):
         return {'admin_state_up': True,
@@ -428,6 +426,26 @@ class FirewallTestCase(BaseFirewallTestCase):
     def test_ingress_tcp_rule(self):
         self._test_rule(self.tester.INGRESS, self.tester.TCP)
 
+    def test_next_port_closed(self):
+        # https://bugs.launchpad.net/neutron/+bug/1611991 was caused by wrong
+        # masking in rules which allow traffic to a port with even port number
+        port = 42
+        for direction in (self.tester.EGRESS, self.tester.INGRESS):
+            sg_rules = [{'ethertype': constants.IPv4,
+                         'direction': direction,
+                         'protocol': constants.PROTO_NAME_TCP,
+                         'source_port_range_min': port,
+                         'source_port_range_max': port}]
+            self._apply_security_group_rules(self.FAKE_SECURITY_GROUP_ID,
+                                             sg_rules)
+
+            self.tester.assert_connection(protocol=self.tester.TCP,
+                                          direction=direction,
+                                          src_port=port)
+            self.tester.assert_no_connection(protocol=self.tester.TCP,
+                                             direction=direction,
+                                             src_port=port + 1)
+
     def test_ingress_udp_rule(self):
         self._test_rule(self.tester.INGRESS, self.tester.UDP)
 
@@ -500,6 +518,7 @@ class FirewallTestCase(BaseFirewallTestCase):
         self._apply_security_group_rules(self.FAKE_SECURITY_GROUP_ID, list())
         self.tester.assert_no_established_connection(**connection)
 
+    @skip_if_firewall('openvswitch')
     def test_preventing_firewall_blink(self):
         direction = self.tester.INGRESS
         sg_rules = [{'ethertype': 'IPv4', 'direction': 'ingress',
@@ -568,9 +587,7 @@ class FirewallTestCase(BaseFirewallTestCase):
 
 
 class FirewallTestCaseIPv6(BaseFirewallTestCase):
-    scenarios = [('OVS Firewall Driver',
-                  {'initialize': 'initialize_ovs',
-                   'firewall_name': 'openvswitch'})]
+    scenarios = BaseFirewallTestCase.scenarios_ovs_fw_interfaces
     ip_cidr = '2001:db8:aaaa::1/64'
 
     def test_icmp_from_specific_address(self):
