@@ -13,10 +13,12 @@
 #    under the License.
 
 import mock
+from neutron_lib import constants
 
 from neutron.api.rpc.handlers import l3_rpc
-from neutron.api.v2 import attributes
-from neutron.common import constants
+from neutron.callbacks import events
+from neutron.callbacks import registry
+from neutron.callbacks import resources
 from neutron.common import topics
 from neutron import context
 from neutron.extensions import external_net
@@ -129,8 +131,8 @@ class L3DvrTestCase(ml2_test_base.ML2TestFramework):
             self.context,
             {'port': {'tenant_id': '',
                       'network_id': network_id,
-                      'mac_address': attributes.ATTR_NOT_SPECIFIED,
-                      'fixed_ips': attributes.ATTR_NOT_SPECIFIED,
+                      'mac_address': constants.ATTR_NOT_SPECIFIED,
+                      'fixed_ips': constants.ATTR_NOT_SPECIFIED,
                       'device_id': self.l3_agent['id'],
                       'device_owner': constants.DEVICE_OWNER_AGENT_GW,
                       portbindings.HOST_ID: '',
@@ -548,6 +550,147 @@ class L3DvrTestCase(ml2_test_base.ML2TestFramework):
                 self.assertEqual(
                     0, l3_notifier.routers_updated_on_host.call_count)
 
+    def test_allowed_addr_pairs_delayed_fip_and_update_arp_entry(self):
+        HOST1 = 'host1'
+        helpers.register_l3_agent(
+            host=HOST1, agent_mode=constants.L3_AGENT_MODE_DVR)
+        HOST2 = 'host2'
+        helpers.register_l3_agent(
+            host=HOST2, agent_mode=constants.L3_AGENT_MODE_DVR)
+        router = self._create_router()
+        private_net1 = self._make_network(self.fmt, 'net1', True)
+        test_allocation_pools = [{'start': '10.1.0.2',
+                                  'end': '10.1.0.20'}]
+        fixed_vrrp_ip = [{'ip_address': '10.1.0.201'}]
+        kwargs = {'arg_list': (external_net.EXTERNAL,),
+                  external_net.EXTERNAL: True}
+        ext_net = self._make_network(self.fmt, '', True, **kwargs)
+        self._make_subnet(
+            self.fmt, ext_net, '10.20.0.1', '10.20.0.0/24',
+            ip_version=4, enable_dhcp=True)
+        # Set gateway to router
+        self.l3_plugin._update_router_gw_info(
+            self.context, router['id'],
+            {'network_id': ext_net['network']['id']})
+        private_subnet1 = self._make_subnet(
+            self.fmt,
+            private_net1,
+            '10.1.0.1',
+            cidr='10.1.0.0/24',
+            ip_version=4,
+            allocation_pools=test_allocation_pools,
+            enable_dhcp=True)
+        vrrp_port = self._make_port(
+            self.fmt,
+            private_net1['network']['id'],
+            fixed_ips=fixed_vrrp_ip)
+        allowed_address_pairs = [
+            {'ip_address': '10.1.0.201',
+             'mac_address': vrrp_port['port']['mac_address']}]
+        with self.port(
+                subnet=private_subnet1,
+                device_owner=DEVICE_OWNER_COMPUTE) as int_port,\
+                self.port(subnet=private_subnet1,
+                          device_owner=DEVICE_OWNER_COMPUTE) as int_port2:
+            self.l3_plugin.add_router_interface(
+                self.context, router['id'],
+                {'subnet_id': private_subnet1['subnet']['id']})
+            with mock.patch.object(self.l3_plugin,
+                                   '_l3_rpc_notifier') as l3_notifier:
+                vm_port = self.core_plugin.update_port(
+                    self.context, int_port['port']['id'],
+                    {'port': {portbindings.HOST_ID: HOST1}})
+                vm_port_mac = vm_port['mac_address']
+                vm_port_fixed_ips = vm_port['fixed_ips']
+                vm_port_subnet_id = vm_port_fixed_ips[0]['subnet_id']
+                vm_arp_table = {
+                    'ip_address': vm_port_fixed_ips[0]['ip_address'],
+                    'mac_address': vm_port_mac,
+                    'subnet_id': vm_port_subnet_id}
+                vm_port2 = self.core_plugin.update_port(
+                    self.context, int_port2['port']['id'],
+                    {'port': {portbindings.HOST_ID: HOST2}})
+                l3_notifier.reset_mock()
+                # Now update the VM port with the allowed_address_pair
+                self.core_plugin.update_port(
+                     self.context, vm_port['id'],
+                     {'port': {
+                         'allowed_address_pairs': allowed_address_pairs}})
+                self.core_plugin.update_port(
+                     self.context, vm_port2['id'],
+                     {'port': {
+                         'allowed_address_pairs': allowed_address_pairs}})
+                self.assertEqual(
+                    0, l3_notifier.routers_updated_on_host.call_count)
+                updated_vm_port1 = self.core_plugin.get_port(
+                    self.context, vm_port['id'])
+                updated_vm_port2 = self.core_plugin.get_port(
+                    self.context, vm_port2['id'])
+                self.assertEqual(4, l3_notifier.add_arp_entry.call_count)
+                expected_allowed_address_pairs = updated_vm_port1.get(
+                    'allowed_address_pairs')
+                self.assertEqual(expected_allowed_address_pairs,
+                                 allowed_address_pairs)
+                expected_allowed_address_pairs_2 = updated_vm_port2.get(
+                    'allowed_address_pairs')
+                self.assertEqual(expected_allowed_address_pairs_2,
+                                 allowed_address_pairs)
+                # Now the VRRP port is attached to the VM port. At this
+                # point, the VRRP port should not have inherited the
+                # port host bindings from the parent VM port.
+                cur_vrrp_port_db = self.core_plugin.get_port(
+                    self.context, vrrp_port['port']['id'])
+                self.assertNotEqual(
+                    cur_vrrp_port_db[portbindings.HOST_ID], HOST1)
+                self.assertNotEqual(
+                    cur_vrrp_port_db[portbindings.HOST_ID], HOST2)
+                # Before we try to associate a floatingip make sure that
+                # only one of the Service port associated with the
+                # allowed_address_pair port is active and the other one
+                # is DOWN
+                mod_vm_port2 = self.core_plugin.update_port(
+                    self.context, updated_vm_port2['id'],
+                    {'port': {
+                        'admin_state_up': False}})
+                self.assertFalse(mod_vm_port2['admin_state_up'])
+                # Next we can try to associate the floatingip to the
+                # VRRP port that is already attached to the VM port
+                l3_notifier.reset_mock()
+                floating_ip = {'floating_network_id': ext_net['network']['id'],
+                               'router_id': router['id'],
+                               'port_id': vrrp_port['port']['id'],
+                               'tenant_id': vrrp_port['port']['tenant_id']}
+                floating_ip = self.l3_plugin.create_floatingip(
+                    self.context, {'floatingip': floating_ip})
+                self.assertEqual(
+                    2, l3_notifier.routers_updated_on_host.call_count)
+                self.assertEqual(3, l3_notifier.add_arp_entry.call_count)
+
+                post_update_vrrp_port_db = self.core_plugin.get_port(
+                    self.context, vrrp_port['port']['id'])
+                vrrp_port_fixed_ips = post_update_vrrp_port_db['fixed_ips']
+                vrrp_port_subnet_id = vrrp_port_fixed_ips[0]['subnet_id']
+                vrrp_arp_table = {
+                    'ip_address': vrrp_port_fixed_ips[0]['ip_address'],
+                    'mac_address': vm_port_mac,
+                    'subnet_id': vrrp_port_subnet_id}
+                vrrp_arp_table1 = {
+                    'ip_address': vrrp_port_fixed_ips[0]['ip_address'],
+                    'mac_address': vrrp_port['port']['mac_address'],
+                    'subnet_id': vrrp_port_subnet_id}
+
+                self.assertEqual(
+                    post_update_vrrp_port_db[portbindings.HOST_ID], HOST1)
+                expected_calls = [
+                        mock.call(self.context,
+                                  router['id'], vrrp_arp_table1),
+                        mock.call(self.context,
+                                  router['id'], vm_arp_table),
+                        mock.call(self.context,
+                                  router['id'], vrrp_arp_table)]
+                l3_notifier.add_arp_entry.assert_has_calls(
+                        expected_calls)
+
     def test_allowed_address_pairs_update_arp_entry(self):
         HOST1 = 'host1'
         helpers.register_l3_agent(
@@ -734,7 +877,7 @@ class L3DvrTestCase(ml2_test_base.ML2TestFramework):
                                  constants.DEVICE_OWNER_LOADBALANCER)
                 # Now change the compute port admin_state_up from True to
                 # False, and see if the vrrp ports device_owner and binding
-                # inheritence reverts back to normal
+                # inheritance reverts back to normal
                 mod_int_port = self.core_plugin.update_port(
                     self.context, cur_int_port['id'],
                     {'port': {
@@ -1392,3 +1535,103 @@ class L3DvrTestCase(ml2_test_base.ML2TestFramework):
             self.context, router['id'])
         self.assertEqual(1, len(agents['agents']))
         self.assertEqual(self.l3_agent['id'], agents['agents'][0]['id'])
+
+    def test_add_router_interface_by_subnet_notifications(self):
+        notif_handler_before = mock.Mock()
+        notif_handler_after = mock.Mock()
+        registry.subscribe(notif_handler_before.callback,
+                           resources.ROUTER_INTERFACE,
+                           events.BEFORE_CREATE)
+        registry.subscribe(notif_handler_after.callback,
+                           resources.ROUTER_INTERFACE,
+                           events.AFTER_CREATE)
+        router = self._create_router()
+        with self.network() as net, \
+                self.subnet(network=net) as subnet:
+            self.l3_plugin.add_router_interface(
+                    self.context, router['id'],
+                    {'subnet_id': subnet['subnet']['id']})
+            kwargs = {'context': self.context, 'router_id': router['id'],
+                      'network_id': net['network']['id']}
+            notif_handler_before.callback.assert_called_once_with(
+                resources.ROUTER_INTERFACE, events.BEFORE_CREATE,
+                mock.ANY, **kwargs)
+            kwargs_after = {'cidrs': mock.ANY,
+                            'context': mock.ANY,
+                            'gateway_ips': mock.ANY,
+                            'interface_info': mock.ANY,
+                            'network_id': None,
+                            'port': mock.ANY,
+                            'port_id': mock.ANY,
+                            'router_id': router['id']}
+            notif_handler_after.callback.assert_called_once_with(
+                resources.ROUTER_INTERFACE, events.AFTER_CREATE,
+                mock.ANY, **kwargs_after)
+
+    def test_add_router_interface_by_port_notifications(self):
+        notif_handler_before = mock.Mock()
+        notif_handler_after = mock.Mock()
+        registry.subscribe(notif_handler_before.callback,
+                           resources.ROUTER_INTERFACE,
+                           events.BEFORE_CREATE)
+        registry.subscribe(notif_handler_after.callback,
+                           resources.ROUTER_INTERFACE,
+                           events.AFTER_CREATE)
+        router = self._create_router()
+        with self.network() as net, \
+                self.subnet(network=net) as subnet, \
+                self.port(subnet=subnet) as port:
+            self.l3_plugin.add_router_interface(
+                    self.context, router['id'],
+                    {'port_id': port['port']['id']})
+            kwargs = {'context': self.context, 'router_id': router['id'],
+                      'network_id': net['network']['id']}
+            notif_handler_before.callback.assert_called_once_with(
+                resources.ROUTER_INTERFACE, events.BEFORE_CREATE,
+                mock.ANY, **kwargs)
+            kwargs_after = {'cidrs': mock.ANY,
+                            'context': mock.ANY,
+                            'gateway_ips': mock.ANY,
+                            'interface_info': mock.ANY,
+                            'network_id': None,
+                            'port': mock.ANY,
+                            'port_id': port['port']['id'],
+                            'router_id': router['id']}
+            notif_handler_after.callback.assert_called_once_with(
+                resources.ROUTER_INTERFACE, events.AFTER_CREATE,
+                mock.ANY, **kwargs_after)
+
+
+class L3DvrTestCaseMigration(L3DvrTestCase):
+    def test_update_router_db_centralized_to_distributed_with_ports(self):
+        with self.subnet() as subnet1:
+            kwargs = {'arg_list': (external_net.EXTERNAL,),
+                      external_net.EXTERNAL: True}
+            with self.network(**kwargs) as ext_net, \
+                    self.subnet(network=ext_net,
+                                cidr='30.0.0.0/24'):
+                router = self._create_router(distributed=False)
+                self.l3_plugin.add_router_interface(
+                    self.context, router['id'],
+                    {'subnet_id': subnet1['subnet']['id']})
+                self.l3_plugin._update_router_gw_info(
+                    self.context, router['id'],
+                    {'network_id': ext_net['network']['id']})
+                self.assertEqual(
+                    0, len(self.l3_plugin._get_snat_sync_interfaces(
+                        self.context, [router['id']])))
+
+                # router needs to be in admin state down in order to be
+                # upgraded to DVR
+                self.l3_plugin.update_router(
+                    self.context, router['id'],
+                    {'router': {'admin_state_up': False}})
+                self.assertFalse(router['distributed'])
+                self.l3_plugin.update_router(
+                    self.context, router['id'],
+                    {'router': {'distributed': True}})
+                router = self.l3_plugin.get_router(self.context, router['id'])
+                self.assertTrue(router['distributed'])
+                self.assertEqual(
+                    1, len(self.l3_plugin._get_snat_sync_interfaces(
+                        self.context, [router['id']])))
