@@ -17,17 +17,19 @@
 import time
 
 import mock
+from neutron_lib import constants as n_const
 from oslo_config import cfg
 from oslo_log import log
 import six
 
 from neutron.agent.common import ip_lib
 from neutron.agent.common import ovs_lib
-from neutron.common import constants as n_const
 from neutron.plugins.common import constants as p_const
 from neutron.plugins.ml2.drivers.openvswitch.agent.common import constants
 from neutron.tests.unit.plugins.ml2.drivers.openvswitch.agent \
     import ovs_test_base
+from neutron.tests.unit.plugins.ml2.drivers.openvswitch.agent \
+    import test_vlanmanager
 
 
 def nonzero(f):
@@ -74,6 +76,11 @@ class TunnelTest(object):
 
     def setUp(self):
         super(TunnelTest, self).setUp()
+        self.useFixture(test_vlanmanager.LocalVlanManagerFixture())
+        conn_patcher = mock.patch(
+            'neutron.agent.ovsdb.native.connection.Connection.start')
+        conn_patcher.start()
+        self.addCleanup(conn_patcher.stop)
         cfg.CONF.set_default('firewall_driver',
                              'neutron.agent.firewall.NoopFirewallDriver',
                              group='SECURITYGROUP')
@@ -89,11 +96,11 @@ class TunnelTest(object):
         self.MAP_TUN_INT_OFPORT = 33333
         self.MAP_TUN_PHY_OFPORT = 44444
 
-        self.LVM = self.mod_agent.LocalVLANMapping(
+        self.LVM_DATA = (
             LV_ID, 'gre', None, LS_ID, VIF_PORTS)
-        self.LVM_FLAT = self.mod_agent.LocalVLANMapping(
+        self.LVM_FLAT_DATA = (
             LV_ID, 'flat', 'net1', LS_ID, VIF_PORTS)
-        self.LVM_VLAN = self.mod_agent.LocalVLANMapping(
+        self.LVM_VLAN_DATA = (
             LV_ID, 'vlan', 'net1', LS_ID, VIF_PORTS)
 
         self.inta = mock.Mock()
@@ -202,6 +209,7 @@ class TunnelTest(object):
 
         self.mock_map_tun_bridge_expected = [
             mock.call.create(),
+            mock.call.set_secure_mode(),
             mock.call.setup_controllers(mock.ANY),
             mock.call.setup_default_table(),
             mock.call.port_exists('phy-%s' % self.MAP_TUN_BRIDGE),
@@ -210,7 +218,7 @@ class TunnelTest(object):
         ]
         self.mock_int_bridge_expected += [
             mock.call.db_get_val('Interface', 'int-%s' % self.MAP_TUN_BRIDGE,
-                                 'type'),
+                                 'type', log_errors=False),
             mock.call.port_exists('int-%s' % self.MAP_TUN_BRIDGE),
             mock.call.add_patch_port('int-%s' % self.MAP_TUN_BRIDGE,
                                      constants.NONEXISTENT_PEER),
@@ -245,7 +253,8 @@ class TunnelTest(object):
             mock.call.add_patch_port('patch-tun', 'patch-int'),
         ]
         self.mock_int_bridge_expected += [
-            mock.call.get_vif_ports(),
+            mock.call.get_vif_ports((ovs_lib.INVALID_OFPORT,
+                                     ovs_lib.UNASSIGNED_OFPORT)),
             mock.call.get_ports_attributes(
                 'Port', columns=['name', 'other_config', 'tag'], ports=[])
         ]
@@ -415,16 +424,16 @@ class TunnelTest(object):
 
         a = self._build_agent()
         a.available_local_vlans = set()
-        a.local_vlan_map[NET_UUID] = self.LVM
+        a.vlan_manager.add(NET_UUID, *self.LVM_DATA)
         a.reclaim_local_vlan(NET_UUID)
-        self.assertIn(self.LVM.vlan, a.available_local_vlans)
+        self.assertIn(self.LVM_DATA[0], a.available_local_vlans)
         self._verify_mock_calls()
 
     def test_reclaim_local_vlan_flat(self):
         self.mock_map_tun_bridge_expected.append(
             mock.call.reclaim_local_vlan(
                 port=self.MAP_TUN_PHY_OFPORT,
-                lvid=self.LVM_FLAT.vlan))
+                lvid=self.LVM_FLAT_DATA[0]))
         self.mock_int_bridge_expected.append(
             mock.call.reclaim_local_vlan(
                 port=self.INT_OFPORT,
@@ -435,16 +444,16 @@ class TunnelTest(object):
         a.int_ofports['net1'] = self.INT_OFPORT
 
         a.available_local_vlans = set()
-        a.local_vlan_map[NET_UUID] = self.LVM_FLAT
+        a.vlan_manager.add(NET_UUID, *self.LVM_FLAT_DATA)
         a.reclaim_local_vlan(NET_UUID)
-        self.assertIn(self.LVM_FLAT.vlan, a.available_local_vlans)
+        self.assertIn(self.LVM_FLAT_DATA[0], a.available_local_vlans)
         self._verify_mock_calls()
 
     def test_reclaim_local_vlan_vlan(self):
         self.mock_map_tun_bridge_expected.append(
             mock.call.reclaim_local_vlan(
                 port=self.MAP_TUN_PHY_OFPORT,
-                lvid=self.LVM_VLAN.vlan))
+                lvid=self.LVM_VLAN_DATA[0]))
         self.mock_int_bridge_expected.append(
             mock.call.reclaim_local_vlan(
                 port=self.INT_OFPORT,
@@ -455,9 +464,9 @@ class TunnelTest(object):
         a.int_ofports['net1'] = self.INT_OFPORT
 
         a.available_local_vlans = set()
-        a.local_vlan_map[NET_UUID] = self.LVM_VLAN
+        a.vlan_manager.add(NET_UUID, *self.LVM_VLAN_DATA)
         a.reclaim_local_vlan(NET_UUID)
-        self.assertIn(self.LVM_VLAN.vlan, a.available_local_vlans)
+        self.assertIn(self.LVM_VLAN_DATA[0], a.available_local_vlans)
         self._verify_mock_calls()
 
     def test_port_bound(self):
@@ -472,7 +481,7 @@ class TunnelTest(object):
                                        vlan_mapping)]
 
         a = self._build_agent()
-        a.local_vlan_map[NET_UUID] = self.LVM
+        a.vlan_manager.add(NET_UUID, *self.LVM_DATA)
         a.local_dvr_map = {}
         self.ovs_bridges[self.INT_BRIDGE].db_get_val.return_value = {}
         a.port_bound(VIF_PORT, NET_UUID, 'gre', None, LS_ID,
@@ -483,7 +492,7 @@ class TunnelTest(object):
         with mock.patch.object(self.mod_agent.OVSNeutronAgent,
                                'reclaim_local_vlan') as reclaim_local_vlan:
             a = self._build_agent()
-            a.local_vlan_map[NET_UUID] = self.LVM
+            a.vlan_manager.add(NET_UUID, *self.LVM_DATA)
             a.port_unbound(VIF_ID, NET_UUID)
 
         reclaim_local_vlan.assert_called_once_with(NET_UUID)
@@ -502,7 +511,7 @@ class TunnelTest(object):
 
         a = self._build_agent()
         a.available_local_vlans = set([LV_ID])
-        a.local_vlan_map[NET_UUID] = self.LVM
+        a.vlan_manager.add(NET_UUID, *self.LVM_DATA)
         self.ovs_bridges[self.INT_BRIDGE].db_get_val.return_value = mock.Mock()
         a.port_dead(VIF_PORT)
         self._verify_mock_calls()
@@ -659,14 +668,15 @@ class TunnelTestUseVethInterco(TunnelTest):
 
         self.mock_map_tun_bridge_expected = [
             mock.call.create(),
+            mock.call.set_secure_mode(),
             mock.call.setup_controllers(mock.ANY),
             mock.call.setup_default_table(),
-            mock.call.add_port(self.intb),
+            mock.call.add_port('phy-%s' % self.MAP_TUN_BRIDGE),
         ]
         self.mock_int_bridge_expected += [
             mock.call.db_get_val('Interface', 'int-%s' % self.MAP_TUN_BRIDGE,
-                                 'type'),
-            mock.call.add_port(self.inta)
+                                 'type', log_errors=False),
+            mock.call.add_port('int-%s' % self.MAP_TUN_BRIDGE)
         ]
 
         self.mock_int_bridge_expected += [
@@ -692,7 +702,8 @@ class TunnelTestUseVethInterco(TunnelTest):
             mock.call.add_patch_port('patch-tun', 'patch-int')
         ]
         self.mock_int_bridge_expected += [
-            mock.call.get_vif_ports(),
+            mock.call.get_vif_ports((ovs_lib.INVALID_OFPORT,
+                                     ovs_lib.UNASSIGNED_OFPORT)),
             mock.call.get_ports_attributes(
                 'Port', columns=['name', 'other_config', 'tag'], ports=[])
         ]
