@@ -13,12 +13,17 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from neutron_lib import constants
 from oslo_config import cfg
 
-from neutron.common import constants
+from neutron.callbacks import events
+from neutron.callbacks import registry
 from neutron.extensions import portbindings
-from neutron.plugins.ml2.drivers.openvswitch.mech_driver \
-    import mech_openvswitch
+from neutron.plugins.ml2 import driver_api as api
+from neutron.plugins.ml2.drivers.openvswitch.agent.common import (
+    constants as a_const)
+from neutron.plugins.ml2.drivers.openvswitch.mech_driver import (
+    mech_openvswitch)
 from neutron.tests.unit.plugins.ml2 import _test_mech_agent as base
 
 
@@ -58,6 +63,19 @@ class OpenvswitchMechanismBaseTestCase(base.AgentMechanismBaseTestCase):
         self.driver = mech_openvswitch.OpenvswitchMechanismDriver()
         self.driver.initialize()
 
+    def test__set_bridge_name_notify(self):
+
+        def fake_callback(resource, event, trigger, **kwargs):
+            trigger('fake-br-name')
+
+        registry.subscribe(fake_callback, a_const.OVS_BRIDGE_NAME,
+                           events.BEFORE_READ)
+        fake_vif_details = {}
+        self.driver._set_bridge_name('foo', fake_vif_details)
+        self.assertEqual(
+            'fake-br-name',
+            fake_vif_details.get(portbindings.VIF_DETAILS_BRIDGE_NAME, ''))
+
 
 class OpenvswitchMechanismSGDisabledBaseTestCase(
     OpenvswitchMechanismBaseTestCase):
@@ -71,6 +89,49 @@ class OpenvswitchMechanismSGDisabledBaseTestCase(
         super(OpenvswitchMechanismSGDisabledBaseTestCase, self).setUp()
 
 
+class OpenvswitchMechanismHybridPlugTestCase(OpenvswitchMechanismBaseTestCase):
+
+    def _make_port_ctx(self, agents):
+        segments = [{api.ID: 'local_segment_id', api.NETWORK_TYPE: 'local'}]
+        return base.FakePortContext(self.AGENT_TYPE, agents, segments,
+                                    vnic_type=self.VNIC_TYPE)
+
+    def test_backward_compat_with_unreporting_agent(self):
+        hybrid = portbindings.OVS_HYBRID_PLUG
+        # agent didn't report so it should be hybrid based on server config
+        context = self._make_port_ctx(self.AGENTS)
+        self.driver.bind_port(context)
+        self.assertTrue(context._bound_vif_details[hybrid])
+        self.driver.vif_details[hybrid] = False
+        context = self._make_port_ctx(self.AGENTS)
+        self.driver.bind_port(context)
+        self.assertFalse(context._bound_vif_details[hybrid])
+
+    def test_hybrid_plug_true_if_agent_requests(self):
+        hybrid = portbindings.OVS_HYBRID_PLUG
+        # set server side default to false and ensure that hybrid becomes
+        # true if requested by the agent
+        self.driver.vif_details[hybrid] = False
+        agents = [{'alive': True,
+                   'configurations': {hybrid: True},
+                   'host': 'host'}]
+        context = self._make_port_ctx(agents)
+        self.driver.bind_port(context)
+        self.assertTrue(context._bound_vif_details[hybrid])
+
+    def test_hybrid_plug_false_if_agent_requests(self):
+        hybrid = portbindings.OVS_HYBRID_PLUG
+        # set server side default to true and ensure that hybrid becomes
+        # false if requested by the agent
+        self.driver.vif_details[hybrid] = True
+        agents = [{'alive': True,
+                   'configurations': {hybrid: False},
+                   'host': 'host'}]
+        context = self._make_port_ctx(agents)
+        self.driver.bind_port(context)
+        self.assertFalse(context._bound_vif_details[hybrid])
+
+
 class OpenvswitchMechanismGenericTestCase(OpenvswitchMechanismBaseTestCase,
                                           base.AgentMechanismGenericTestCase):
     pass
